/*
 * Copyright (C) 2001 and 2002 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ifndef _CTMPL_AVLTREE_C_
#define _CTMPL_AVLTREE_C_
#define avt_root(x)     ((x)->root)
#define avt_count(x)    ((x)->count)
#define avt_isempty(x)  (avt_count(x) == 0)
#define avtn_key(t)     ((t)->data.first)
#define avtn_value(t)   ((t)->data.second)
#define avtn_left(t)    ((t)->left)
#define avtn_right(t)   ((t)->right)
#define avtn_parent(t)  ((t)->parent)
#define avtn_height(t)  ((t)->height)

#define avt_max(a, b)   (((a) > (b))? (a): (b))
#define avt_min(a, b)   (((a) > (b))? (b): (a))
#define AVTHEIGHT(x) ((x)? avtn_height(x): -1)
#endif	/* _CTMPL_AVLTREE_C_ */

#include "ctmpl/memory.h"
#include "ctmpl/memory.c"

static CTMPL_INLINE void PKG_NAME_(avt_set_ctl)(PKG_NAME_(avt));
static void PKG_NAME_(avt_copy_recursive)(PKG_NAME_(avtnode) k, 
					 PKG_NAME_(avt) t,
					 PKG_NAME_(avtnode_duplicate_func_t) f);
static CTMPL_INLINE void 
PKG_NAME_(avtnode_delete)(PKG_NAME_(avt) avt,
			 PKG_NAME_(avtnode) avtn,
			 PKG_NAME_(avtnode_access_func_t) df);
static PKG_NAME_(avtnode) PKG_NAME_(avtnode_find_min)(PKG_NAME_(avtnode) k);
static PKG_NAME_(avtnode) PKG_NAME_(avtnode_find_max)(PKG_NAME_(avtnode) k);
static PKG_NAME_(avtnode) PKG_NAME_(avtnode_find)(PKG_NAME_(avt) avt,
					     PKG_NAME_(avtnode) k, 
					     PKG_NAME_(avtkey_t) key);
static PKG_NAME_(avtnode) PKG_NAME_(avtnode_insert)(PKG_NAME_(avt) avt,
					       PKG_NAME_(avtnode) avtn1,
					       PKG_NAME_(avtnode) avtn2);
static void PKG_NAME_(avtnode_remove)(PKG_NAME_(avt) avt,
				     PKG_NAME_(avtnode) k);

static PKG_NAME_(avtnode) PKG_NAME_(avtnode_balance)(PKG_NAME_(avt) avt,
				       		     PKG_NAME_(avtnode) avtn);
static void PKG_NAME_(avtnode_balance_recursive)(PKG_NAME_(avt) avt, 
				     		 PKG_NAME_(avtnode) avtn);

static PKG_NAME_(avtnode) PKG_NAME_(avtnode_single_rotate_with_left)(PKG_NAME_(avtnode) k);
static PKG_NAME_(avtnode) PKG_NAME_(avtnode_single_rotate_with_right)(PKG_NAME_(avtnode) k);
static PKG_NAME_(avtnode) PKG_NAME_(avtnode_double_rotate_with_left)(PKG_NAME_(avtnode) k);
static PKG_NAME_(avtnode) PKG_NAME_(avtnode_double_rotate_with_right)(PKG_NAME_(avtnode) k);


CTMPL_STATIC CTMPL_INLINE PKG_NAME_(avt) 
PKG_NAME_(avt_new)(PKG_NAME_(avt_cmpkey_func_t) ck)
{
	return PKG_NAME_(avt_new_by)(ck, NULL, NULL);
}

CTMPL_STATIC CTMPL_INLINE PKG_NAME_(avt) 
PKG_NAME_(avt_new_by)(PKG_NAME_(avt_cmpkey_func_t) cmpkey_func,
		     m_alloc_func_t af,	
		     m_free_func_t ff)
{
	PKG_NAME_(avt) ret;
	ctmpl_assert(cmpkey_func != NULL);

	/* default alloc/free function */
	if(!af)	 af = PKG_NAME_(m_alloc);
	if(!ff)	 ff = PKG_NAME_(m_free);

	ret = CTMPL_MALLOC(af, PKG_NAME_STRING "avt", 
			   sizeof(struct PKG_NAME_(avt)));
	ret->root = NULL;
	ret->cmpkey = cmpkey_func;
	ret->alloc = af;
	ret->free = ff;
	PKG_NAME_(avt_set_ctl)(ret);
	return ret;
}


static CTMPL_INLINE void 
PKG_NAME_(avt_foreach_recursive)(PKG_NAME_(avtnode) tn, 
				PKG_NAME_(avtnode_foreach_func_t) f,
		      		void *arg)
{
	if(!tn)
		return;
	PKG_NAME_(avt_foreach_recursive)(avtn_left(tn), f, arg);	
	if(f)
		f(avtn_key(tn), avtn_value(tn), arg);
	PKG_NAME_(avt_foreach_recursive)(avtn_right(tn), f, arg);	
}

static CTMPL_INLINE void 
PKG_NAME_(avt_foreach)(PKG_NAME_(avt) t, 
		      PKG_NAME_(avtnode_foreach_func_t) f,
		      void *arg)
{
	ctmpl_assert(t != NULL);
	PKG_NAME_(avt_foreach_recursive)(avt_root(t), f, arg);
}


static CTMPL_INLINE iterator_cnt_t 
PKG_NAME_(avt_count)(PKG_NAME_(avt) avt){ return avt_count(avt); }

static CTMPL_INLINE void 
PKG_NAME_(avt_clear_by)(PKG_NAME_(avt) t, 
		       PKG_NAME_(avtnode_access_func_t) df)
{
	PKG_NAME_(avtnode_delete)(t, avt_root(t), df);
	avt_root(t) = NULL;
	avt_count(t) = 0;
}

static CTMPL_INLINE void 
PKG_NAME_(avt_delete_by)(PKG_NAME_(avt) t, 
			PKG_NAME_(avtnode_access_func_t) df)
{
	PKG_NAME_(avt_clear_by)(t, df);
	CTMPL_FREE(t->free, PKG_NAME_STRING "avt", t);
}


static CTMPL_INLINE PKG_NAME_(avt) 
PKG_NAME_(avt_dup_by)(PKG_NAME_(avt) avt, 
		     PKG_NAME_(avtnode_duplicate_func_t) f)
{
	PKG_NAME_(avt) ret;
	ctmpl_assert(avt);

	ret = CTMPL_MALLOC(avt->alloc, PKG_NAME_STRING "avt", 
			   sizeof(struct PKG_NAME_(avt)));
	ret->root = NULL;
	ret->cmpkey = avt->cmpkey;
	ret->alloc = avt->alloc;
	ret->free = avt->free;
	PKG_NAME_(avt_set_ctl)(ret);
	PKG_NAME_(avt_copy_recursive)(avt_root(avt), ret, f);
	return ret;
}


static CTMPL_INLINE void
PKG_NAME_(avt_put)(PKG_NAME_(avt) t, 
		  PKG_NAME_(avtkey_t) k, 
		  PKG_NAME_(avtval_t) v)
{
	PKG_NAME_(avtnode) node;
	ctmpl_assert(t != NULL);
	
#ifndef PKG_USE_MULTI /* not multi map */
	/* check value whose key is same */
	node = PKG_NAME_(avtnode_find)(t, avt_root(t), k);

	/* value of same key already exists in table, swap */
	if(node){
		avtn_key(node) = k;
		avtn_value(node) = v;
		return;
	}
#endif /* PKG_USE_MULTI  */

	/* new node */
	node = CTMPL_MALLOC(t->alloc,
		PKG_NAME_STRING "avtnode", sizeof(struct PKG_NAME_(avtnode)));
	avtn_key(node) = k;
	avtn_value(node) = v;
	avtn_parent(node) = NULL;
	avtn_left(node) = NULL;
	avtn_right(node) = NULL;
	avtn_height(node) = 0;

	avt_root(t) = PKG_NAME_(avtnode_insert)(t, avt_root(t), node);
	return;
}

static PKG_NAME_(avt_iterator)
PKG_NAME_(avt_get)(PKG_NAME_(avt) t, 
                  PKG_NAME_(avtkey_t) k)
{
	PKG_NAME_(avt_iterator) ret;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
#ifndef PKG_USE_MULTI /* not multi map */
	iterator_subtype(ret) = 0;
#else 
	iterator_subtype(ret) = 1;
#endif /* PKG_USE_MULTI */
	iterator_aggregate(ret) = t;
	ctmpl_assert(t != NULL);
	
	iterator_item(ret) = PKG_NAME_(avtnode_find)(t, avt_root(t), k);
	return ret;
}

static CTMPL_INLINE void 
PKG_NAME_(avtnode_delete)(PKG_NAME_(avt) avt,
			 PKG_NAME_(avtnode) avtn,
			 PKG_NAME_(avtnode_access_func_t) df)
{
	if(!avtn)
		return;

	PKG_NAME_(avtnode_delete)(avt, avtn_left(avtn), df);
	PKG_NAME_(avtnode_delete)(avt, avtn_right(avtn), df);
	if(df)		(*df)(avtn_key(avtn), avtn_value(avtn));
	CTMPL_FREE(avt->free, PKG_NAME_STRING "avtnode", avtn);
}

static PKG_NAME_(avtnode) 
PKG_NAME_(avtnode_find_min)(PKG_NAME_(avtnode) k)
{
	if(!k)	return NULL;
	else if(avtn_left(k))
		return PKG_NAME_(avtnode_find_min)(avtn_left(k));
	return k;
}

static PKG_NAME_(avtnode) 
PKG_NAME_(avtnode_find_max)(PKG_NAME_(avtnode) k)
{
	if(!k)	return NULL;
	else if(avtn_right(k))
		return PKG_NAME_(avtnode_find_max)(avtn_right(k));
	return k;
}

static PKG_NAME_(avtnode) 
PKG_NAME_(avtnode_find)(PKG_NAME_(avt) avt, 
		   PKG_NAME_(avtnode) avtn, 
		   PKG_NAME_(avtkey_t) key)
{
	int result;
	if(!avtn)
		return NULL;
	result = (*avt->cmpkey)(key, avtn_key(avtn));
	if(result < 0)
		return PKG_NAME_(avtnode_find)(avt, avtn_left(avtn), key);
	else if(result > 0)
		return PKG_NAME_(avtnode_find)(avt, avtn_right(avtn), key);
#ifdef PKG_USE_MULTI
	{
		/* could be more optimized */
		PKG_NAME_(avtnode) ret = NULL;
		ret = PKG_NAME_(avtnode_find)(avt, avtn_left(avtn), key);
		if(ret)
			return ret;
	}
#endif /* PKG_USE_MULTI */
	return avtn;
}

static PKG_NAME_(avtnode) PKG_NAME_(avtnode_insert)(PKG_NAME_(avt) avt,
					      PKG_NAME_(avtnode) avtn1,
					      PKG_NAME_(avtnode) avtn2)
{
	int result;

	/* no parent node */
	if(!avtn1){
		++avt_count(avt);
		avtn_height(avtn2) = 0;
		return avtn2;
	}

	result = (*avt->cmpkey)(avtn_key(avtn2), avtn_key(avtn1));
	if(result < 0){
		avtn_left(avtn1) = 
			PKG_NAME_(avtnode_insert)(avt, avtn_left(avtn1), avtn2);
		avtn_parent(avtn_left(avtn1)) = avtn1;
	}
#ifndef PKG_USE_MULTI /* not multi map */
	else if(result > 0){
#else  /* PKG_USE_MULTI */
	else if(result >= 0){
#endif /* PKG_USE_MULTI */
		avtn_right(avtn1) = 
			PKG_NAME_(avtnode_insert)(avt, avtn_right(avtn1), avtn2);
		avtn_parent(avtn_right(avtn1)) = avtn1;
	}
	else /* already same node is here */
		ctmpl_assert("Already same node exists!" == NULL);

	return PKG_NAME_(avtnode_balance)(avt, avtn1);
}


static void PKG_NAME_(avtnode_remove)(PKG_NAME_(avt) avt,
				     PKG_NAME_(avtnode) avtn)
{
	PKG_NAME_(avtnode) tmp = NULL;
	/* two children */
	if(avtn_left(avtn) && avtn_right(avtn)){
		/* minimum node from right tree */
		tmp = PKG_NAME_(avtnode_find_min)(avtn_right(avtn));
		/* remove this node once */
		PKG_NAME_(avtnode_remove)(avt, tmp);
	
		/* swap with avtn */
		avtn_parent(tmp) = avtn_parent(avtn);
		avtn_left(tmp) = avtn_left(avtn);
		avtn_right(tmp) = avtn_right(avtn);
		avtn_height(tmp) = avtn_height(avtn);

		if(avtn_parent(avtn)){
			if(avtn_left(avtn_parent(avtn)) == avtn)
				avtn_left(avtn_parent(avtn)) = tmp;
			else 
				avtn_right(avtn_parent(avtn)) = tmp;
		}
		if(avtn_left(avtn))
			avtn_parent(avtn_left(avtn)) = tmp;
		if(avtn_right(avtn))
			avtn_parent(avtn_right(avtn)) = tmp;
	} 
	else {
		/* change parent pointer of child node */
		if(avtn_left(avtn)){
			avtn_parent(avtn_left(avtn)) = avtn_parent(avtn);
			tmp = avtn_left(avtn);
		}
		else if(avtn_right(avtn)){
			avtn_parent(avtn_right(avtn)) = avtn_parent(avtn);
			tmp = avtn_right(avtn);
		}

		/* change child pointer of parent node */
		if(avtn_parent(avtn)){
			if(avtn_left(avtn_parent(avtn)) == avtn)
				avtn_left(avtn_parent(avtn)) = tmp;
			else 
				avtn_right(avtn_parent(avtn)) = tmp;
		}
		--avt_count(avt);
		PKG_NAME_(avtnode_balance_recursive)(avt, avtn_parent(avtn));
	}
	if(avt_root(avt) == avtn)
		avt_root(avt) = tmp;
	avtn_parent(avtn) = NULL;
	avtn_left(avtn) = NULL;
	avtn_right(avtn) = NULL;
}

static void
PKG_NAME_(avt_copy_recursive)(PKG_NAME_(avtnode) k,
			     PKG_NAME_(avt) t,
			     PKG_NAME_(avtnode_duplicate_func_t) f)
{
	struct PKG_NAME_(avt_iterator_data) dup;
	if(k == NULL)
		return;
	if(f){
		dup = f(avtn_key(k), avtn_value(k));
		PKG_NAME_(avt_put)(t, dup.first, dup.second);
	}
	else
		PKG_NAME_(avt_put)(t, avtn_key(k), avtn_value(k));
	PKG_NAME_(avt_copy_recursive)(avtn_left(k), t, f);
	PKG_NAME_(avt_copy_recursive)(avtn_right(k), t, f);
}


static PKG_NAME_(avtnode) 
PKG_NAME_(avtnode_itr_next)(PKG_NAME_(avt), PKG_NAME_(avtnode));
static PKG_NAME_(avtnode) 
PKG_NAME_(avtnode_itr_prev)(PKG_NAME_(avt), PKG_NAME_(avtnode));

static PKG_NAME_(avt_iterator) 
PKG_NAME_(avt_itr_first)(PKG_NAME_(avt) avt)
{
	PKG_NAME_(avt_iterator) ret;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = avt;
	iterator_item(ret) = PKG_NAME_(avtnode_find_min)(avt_root(avt));
	return ret;
}
static PKG_NAME_(avt_iterator) 
PKG_NAME_(avt_itr_last)(PKG_NAME_(avt) avt)
{
	PKG_NAME_(avt_iterator) ret;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = avt;
	iterator_item(ret) = PKG_NAME_(avtnode_find_max)(avt_root(avt));
	return ret;
}

static PKG_NAME_(avt_iterator) 
PKG_NAME_(avt_itr_at)(PKG_NAME_(avt) avt, iterator_pos_t idx)
{
	int prev = 0;
	PKG_NAME_(avt_iterator) ret;
	PKG_NAME_(avtnode) avtn;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = avt;
	
	if(idx < 0){
		++prev;
		idx = -idx - 1;
	}

	avtn = (prev? PKG_NAME_(avtnode_find_max)(avt_root(avt)):
		     PKG_NAME_(avtnode_find_min)(avt_root(avt)));

	while(idx){
		avtn = (prev? PKG_NAME_(avtnode_itr_prev)(avt, avtn):
			     PKG_NAME_(avtnode_itr_next)(avt, avtn));
		--idx;
	}
	iterator_item(ret) = avtn;
	return ret;
}

static PKG_NAME_(avt_iterator) 
PKG_NAME_(avt_itr_next)(PKG_NAME_(avt_iterator) itr)
{
	PKG_NAME_(avt) avt = iterator_aggregate(itr);
	PKG_NAME_(avt_iterator) ret;
	iterator_settype(ret, iterator_type(itr));
	iterator_subtype(ret) = iterator_subtype(itr);
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = NULL;

	if(iterator_is_end(itr))
		return itr;
	iterator_item(ret) = PKG_NAME_(avtnode_itr_next)
			(iterator_aggregate(ret), iterator_item(itr));

	/* multi map iterator */
	if(iterator_subtype(ret) && iterator_item(ret) &&
	   (*avt->cmpkey)(avtn_key(iterator_item(itr)), 
		         avtn_key(iterator_item(ret))) != 0)
		iterator_item(ret) = NULL;
	return ret;
}
static PKG_NAME_(avt_iterator) 
PKG_NAME_(avt_itr_prev)(PKG_NAME_(avt_iterator) itr)
{
	PKG_NAME_(avt) avt = iterator_aggregate(itr);
	PKG_NAME_(avt_iterator) ret;
	iterator_settype(ret, iterator_type(itr));
	iterator_subtype(ret) = iterator_subtype(itr);
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = NULL;

	if(iterator_is_end(itr))
		return itr;
	iterator_item(ret) = PKG_NAME_(avtnode_itr_prev)
			(iterator_aggregate(ret), iterator_item(itr));

	/* multi map iterator */
	if(iterator_subtype(ret) && iterator_item(ret) &&
	   (*avt->cmpkey)(avtn_key(iterator_item(itr)), 
		         avtn_key(iterator_item(ret))) != 0)
		iterator_item(ret) = NULL;
	return ret;
}

static PKG_NAME_(avt_iterator) 
PKG_NAME_(avt_itr_erase_gonext)(PKG_NAME_(avt_iterator) itr)
{
	PKG_NAME_(avt) t;
	PKG_NAME_(avtnode) n;
	PKG_NAME_(avt_iterator) ret;

	if(iterator_is_end(itr))
		return itr;

	ret = PKG_NAME_(avt_itr_next)(itr);
	t = iterator_aggregate(itr);
	n = iterator_item(itr);
	PKG_NAME_(avtnode_remove)(t, n);
	CTMPL_FREE(t->free, PKG_NAME_STRING "avtnode", n);
	return ret;
}

static PKG_NAME_(avt_iterator) 
PKG_NAME_(avt_itr_erase_goprev) (PKG_NAME_(avt_iterator) itr)
{
	PKG_NAME_(avt) t;
	PKG_NAME_(avtnode) n;
	PKG_NAME_(avt_iterator) ret;

	if(iterator_is_end(itr))
		return itr;

	ret = PKG_NAME_(avt_itr_prev)(itr);
	t = iterator_aggregate(itr);
	n = iterator_item(itr);
	PKG_NAME_(avtnode_remove)(t, n);
	CTMPL_FREE(t->free, PKG_NAME_STRING "avtnode", n);
	return ret;
}

#if 0
static PKG_NAME_(avt_iterator) 
PKG_NAME_(avt_itr_next)(PKG_NAME_(avt_iterator) itr)
{
	PKG_NAME_(avt_iterator) ret;
	iterator_settype(ret, iterator_type(itr));
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = NULL;

	if(iterator_is_end(itr))
		return ret;
	iterator_item(ret) = PKG_NAME_(avtnode_itr_next)
			(iterator_aggregate(ret), iterator_item(itr));
	return ret;
}
static PKG_NAME_(avt_iterator) 
PKG_NAME_(avt_itr_prev)(PKG_NAME_(avt_iterator) itr)
{
	PKG_NAME_(avt_iterator) ret;
	iterator_settype(ret, iterator_type(itr));
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = NULL;

	if(iterator_is_end(itr))
		return ret;
	iterator_item(ret) = PKG_NAME_(avtnode_itr_prev)
			(iterator_aggregate(ret), iterator_item(itr));
	return ret;
}

static PKG_NAME_(avt_iterator) 
PKG_NAME_(avt_itr_erase_gonext)(PKG_NAME_(avt_iterator) itr)
{
	PKG_NAME_(avt) t;
	PKG_NAME_(avtnode) n;
	PKG_NAME_(avt_iterator) ret;
	iterator_settype(ret, iterator_type(itr));
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = NULL;

	if(iterator_is_end(itr))
		return ret;

	t = iterator_aggregate(itr);
	n = iterator_item(itr);
	iterator_item(ret) = PKG_NAME_(avtnode_itr_next)(t, n); 
	PKG_NAME_(avtnode_remove)(t, n);
	CTMPL_FREE(t->free, PKG_NAME_STRING "avtnode", n);
	return ret;
}

static PKG_NAME_(avt_iterator) 
PKG_NAME_(avt_itr_erase_goprev) (PKG_NAME_(avt_iterator) itr)
{
	PKG_NAME_(avt) t;
	PKG_NAME_(avtnode) n;
	PKG_NAME_(avt_iterator) ret;
	iterator_settype(ret, iterator_type(itr));
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = NULL;

	if(iterator_is_end(itr))
		return ret;

	t = iterator_aggregate(itr);
	n = iterator_item(itr);

	iterator_item(ret) = PKG_NAME_(avtnode_itr_prev)(t, n);
	PKG_NAME_(avtnode_remove)(t, n);
	CTMPL_FREE(t->free, PKG_NAME_STRING "avtnode", n);
	return ret;
}
#endif 

static int
PKG_NAME_(avt_itr_is_equal)(PKG_NAME_(avt_iterator) x, 
			    PKG_NAME_(avt_iterator) y)
{ return CTMPL_ITR_IS_EQUAL(x, y); }

static PKG_NAME_(avt_iterator_data) 
PKG_NAME_(avt_itr_retrieve)(PKG_NAME_(avt_iterator) itr)
{
	return (PKG_NAME_(avt_iterator_data))iterator_item(itr);
}

static PKG_NAME_(avtnode) 
PKG_NAME_(avtnode_itr_next)(PKG_NAME_(avt) avt, PKG_NAME_(avtnode) avtn)
{
	PKG_NAME_(avtnode) pp, pq;
	if(!avtn)
		return NULL;

	/* has right node */
	if(avtn_right(avtn))
		return PKG_NAME_(avtnode_find_min)(avtn_right(avtn));

	/* this node is parent's left node */
	if(avtn_parent(avtn) && avtn_left(avtn_parent(avtn)) == avtn) 
		return avtn_parent(avtn);

	pp = pq = avtn;
	while((pp = avtn_parent(pq)) != NULL){
		/* this node is parent's left node */
		if(avtn_left(avtn_parent(pq)) == pq) 
			return pp;
		pq = pp;
	}
	return NULL;
}

static PKG_NAME_(avtnode) 
PKG_NAME_(avtnode_itr_prev)(PKG_NAME_(avt) avt, PKG_NAME_(avtnode) avtn)
{
	PKG_NAME_(avtnode) pp, pq;
	if(!avtn)
		return NULL;

	/* has left node */
	if(avtn_left(avtn))
		return PKG_NAME_(avtnode_find_max)(avtn_left(avtn));

	/* this node is parent's right node */
	if(avtn_parent(avtn) && avtn_right(avtn_parent(avtn)) == avtn) 
		return avtn_parent(avtn);

	pp = pq = avtn;
	while((pp = avtn_parent(pq)) != NULL){
		/* this node is parent's right node */
		if(avtn_right(avtn_parent(pq)) == pq) 
			return pp;
		pq = pp;
	}
	return NULL;
}

static PKG_NAME_(avt_ctl) PKG_NAME_(avt_ctl_sw) = 
{
	{-1, -1, NULL, NULL},
	PKG_NAME_(avt_delete_by),
	PKG_NAME_(avt_clear_by),
	PKG_NAME_(avt_dup_by),
	PKG_NAME_(avt_foreach),
	PKG_NAME_(avt_count),
	PKG_NAME_(avt_itr_first),
	PKG_NAME_(avt_itr_last),
	PKG_NAME_(avt_itr_at),
	PKG_NAME_(avt_itr_next),
	PKG_NAME_(avt_itr_prev),
	PKG_NAME_(avt_itr_erase_gonext),
	PKG_NAME_(avt_itr_erase_goprev),
	PKG_NAME_(avt_itr_is_equal),
	PKG_NAME_(avt_itr_retrieve),
	PKG_NAME_(avt_get),
	PKG_NAME_(avt_put)
};

static CTMPL_INLINE void 
PKG_NAME_(avt_set_ctl)(PKG_NAME_(avt) avtp)
{
	avtp->ctl = &PKG_NAME_(avt_ctl_sw);
}

/*
 * this function was recursive but it's slower than loop.
 * so change to use loop.
 */
static void 
PKG_NAME_(avtnode_balance_recursive)(PKG_NAME_(avt) avt, 
				     PKG_NAME_(avtnode) avtn)
{
	PKG_NAME_(avtnode) *ptr = NULL;
	
	while(avtn){
		/* this node is root */
		if(!avtn_parent(avtn))
			ptr = &avt_root(avt);
		else  /* not root */
			ptr = (avtn_left(avtn_parent(avtn)) == avtn ?
				&avtn_left(avtn_parent(avtn)):
				&avtn_right(avtn_parent(avtn)));

		/* re-balance */
		*ptr = PKG_NAME_(avtnode_balance)(avt, avtn);
		avtn = avtn_parent(*ptr);
	}
}

static PKG_NAME_(avtnode)
PKG_NAME_(avtnode_balance)(PKG_NAME_(avt) avt, PKG_NAME_(avtnode) avtn)
{
	int hl, hr, sub, isLeft;
	PKG_NAME_(avtnode) child = NULL;
	typedef PKG_NAME_(avtnode) (*rotate_t)(PKG_NAME_(avtnode) n);
	rotate_t rotate;

	if(avtn == NULL)
		return NULL;

	hl = AVTHEIGHT(avtn_left(avtn));
	hr = AVTHEIGHT(avtn_right(avtn));
	
	sub = hl - hr;

	/* don't need to balance */
	if(sub > -2 && sub < 2){
		/* recalculate height */
		avtn_height(avtn) = avt_max(AVTHEIGHT(avtn_left(avtn)),
				    AVTHEIGHT(avtn_right(avtn))) + 1;
		return avtn;
	}

	/* must be 2 or -2 */
	ctmpl_assert(sub == -2 || sub == 2); 

	isLeft = (sub > 0);
	child = (isLeft  ? avtn_left(avtn): avtn_right(avtn));

	if(isLeft){
		if((AVTHEIGHT(avtn_right(child)) 
			- AVTHEIGHT(avtn_left(child))) == 1)
			rotate = PKG_NAME_(avtnode_double_rotate_with_left);
		else 
			rotate = PKG_NAME_(avtnode_single_rotate_with_left);
	}
	else {
		if((AVTHEIGHT(avtn_left(child)) 
			- AVTHEIGHT(avtn_right(child))) == 1)
			rotate = PKG_NAME_(avtnode_double_rotate_with_right);
		else 
			rotate = PKG_NAME_(avtnode_single_rotate_with_right);
	}
	avtn = (*rotate)(avtn);
	avtn_height(avtn) = avt_max(AVTHEIGHT(avtn_left(avtn)),
				    AVTHEIGHT(avtn_right(avtn))) + 1;
	/* new root */
	return avtn;
}


static PKG_NAME_(avtnode)
PKG_NAME_(avtnode_single_rotate_with_left)(PKG_NAME_(avtnode) n)
{
	PKG_NAME_(avtnode) n1 = n, n2 = avtn_left(n1), root = avtn_parent(n);

	avtn_left(n1) = avtn_right(n2); 
	if(avtn_right(n2))
		avtn_parent(avtn_right(n2)) = n1;
	avtn_right(n2) = n1;
	avtn_parent(n1) = n2;

	avtn_height(n1) = avt_max(AVTHEIGHT(avtn_left(n1)),  
				  AVTHEIGHT(avtn_right(n1))) + 1;
	avtn_height(n2) = avt_max(AVTHEIGHT(avtn_left(n2)),  
				  AVTHEIGHT(n1)) + 1;
	avtn_parent(n2) = root;
	return n2; /* new root node */
}

static PKG_NAME_(avtnode) 
PKG_NAME_(avtnode_single_rotate_with_right)(PKG_NAME_(avtnode) n)
{
	PKG_NAME_(avtnode) n1 = n, n2 = avtn_right(n1), root = avtn_parent(n);

	avtn_right(n1) = avtn_left(n2); 
	if(avtn_left(n2))
		avtn_parent(avtn_left(n2)) = n1;
	avtn_left(n2) = n1;
	avtn_parent(n1) = n2;

	avtn_height(n1) = avt_max(AVTHEIGHT(avtn_right(n1)),  
				  AVTHEIGHT(avtn_left(n1))) + 1;
	avtn_height(n2) = avt_max(AVTHEIGHT(avtn_right(n2)),  
				  AVTHEIGHT(n1)) + 1;
	avtn_parent(n2) = root;
	return n2; /* new root node */
}

static PKG_NAME_(avtnode) 
PKG_NAME_(avtnode_double_rotate_with_left)(PKG_NAME_(avtnode) n)
{
	avtn_left(n) = 
	    PKG_NAME_(avtnode_single_rotate_with_right)(avtn_left(n));
	return PKG_NAME_(avtnode_single_rotate_with_left)(n);
}
static PKG_NAME_(avtnode) 
PKG_NAME_(avtnode_double_rotate_with_right)(PKG_NAME_(avtnode) n)
{
	avtn_right(n) = 
	    PKG_NAME_(avtnode_single_rotate_with_left)(avtn_right(n));
	return PKG_NAME_(avtnode_single_rotate_with_right)(n);
}

