/*
 * Copyright (C) 2001 and 2002 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef _CTMPL_AVLTREE_SET_C_
#define _CTMPL_AVLTREE_SET_C_
#define avts_root(x)	avt_root((x)->avt)
#define avts_count(x)	avt_count((x)->avt)
#define avts_isempty(x)	(avts_count(x) == 0)
#define avtsn_key(t)	((t)->data.first)
#define avtsn_value(t)	((t)->data.second)
#define avtsn_left(t)	((t)->left)
#define avtsn_right(t)	((t)->right)
#define avtsn_parent(t)	((t)->parent)
#endif	/* _CTMPL_AVLTREE_C_ */

#include "ctmpl/avltree.c"

static CTMPL_INLINE void PKG_NAME_(avts_set_ctl)(PKG_NAME_(avts));
static void PKG_NAME_(avts_copy_recursive)(PKG_NAME_(avtsnode) k, 
					  PKG_NAME_(avts) t,
					  PKG_NAME_(avtsnode_duplicate_func_t) f);
static CTMPL_INLINE void 
PKG_NAME_(avtsnode_delete)(PKG_NAME_(avts) avts,
			  PKG_NAME_(avtsnode) avtsn,
			  PKG_NAME_(avtsnode_access_func_t) df);

static CTMPL_INLINE void PKG_NAME_(avts_set_ctl)(PKG_NAME_(avts));

CTMPL_STATIC CTMPL_INLINE PKG_NAME_(avts) 
PKG_NAME_(avts_new)(PKG_NAME_(avts_cmpval_func_t) ck)
{
	return PKG_NAME_(avts_new_by)(ck, NULL, NULL);
}

CTMPL_STATIC CTMPL_INLINE PKG_NAME_(avts) 
PKG_NAME_(avts_new_by)(PKG_NAME_(avts_cmpval_func_t) cmp_func,
		     m_alloc_func_t af,	m_free_func_t ff)
{
	PKG_NAME_(avts) ret;
	ctmpl_assert(cmp_func != NULL);

	/* default alloc/free function */
	if(!af)	 af = PKG_NAME_(m_alloc);
	if(!ff)	 ff = PKG_NAME_(m_free);

	ret = CTMPL_MALLOC(af, PKG_NAME_STRING "avts", 
			   sizeof(struct PKG_NAME_(avts)));
	ret->avt = PKG_NAME_(avt_new_by)(cmp_func, af, ff);
	PKG_NAME_(avts_set_ctl)(ret);
	return ret;
}

static CTMPL_INLINE iterator_cnt_t 
PKG_NAME_(avts_count)(PKG_NAME_(avts) avts){ return avts_count(avts); }

static CTMPL_INLINE void
PKG_NAME_(avts_put)(PKG_NAME_(avts) t, 
		   PKG_NAME_(avtsval_t) v)
{
	PKG_NAME_(avt_put)(t->avt, v, v);
}

static PKG_NAME_(avts_iterator) 
PKG_NAME_(avts_get)(PKG_NAME_(avts) t, 
		   PKG_NAME_(avtsval_t) v)
{
	PKG_NAME_(avt_iterator) avtitr;
	PKG_NAME_(avts_iterator) ret;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_aggregate(ret) = t;
	ctmpl_assert(t != NULL);
	
	avtitr = iterator_find(t->avt, v);
	iterator_subtype(ret) = iterator_subtype(avtitr);
	iterator_item(ret) = iterator_item(avtitr);
	return ret;
}

static CTMPL_INLINE void 
PKG_NAME_(avts_clear_by)(PKG_NAME_(avts) t, 
		        PKG_NAME_(avtsnode_access_func_t) df)
{
	PKG_NAME_(avtsnode_delete)(t, avts_root(t), df);
	avts_root(t) = NULL;
	avts_count(t) = 0;
}

static CTMPL_INLINE void 
PKG_NAME_(avts_delete_by)(PKG_NAME_(avts) t, 
			PKG_NAME_(avtsnode_access_func_t) df)
{
	m_free_func_t free_func = t->avt->free;
	PKG_NAME_(avts_clear_by)(t, df);
	PKG_NAME_(avt_delete_by)(t->avt, NULL);
	CTMPL_FREE(free_func, PKG_NAME_STRING "avts", t);
}


static CTMPL_INLINE PKG_NAME_(avts) 
PKG_NAME_(avts_dup_by)(PKG_NAME_(avts) avts, 
		      PKG_NAME_(avtsnode_duplicate_func_t) f)
{
	PKG_NAME_(avts) ret;
	ctmpl_assert(avts);

	ret = PKG_NAME_(avts_new_by)(avts->avt->cmpkey,
				    avts->avt->alloc,
			  	    avts->avt->free);
	PKG_NAME_(avts_copy_recursive)(avts_root(avts), ret, f);
	return ret;
}

static CTMPL_INLINE void 
PKG_NAME_(avts_foreach_recursive)(PKG_NAME_(avtsnode) tn, 
				 PKG_NAME_(avtsnode_foreach_func_t) f,
		      		 void *arg)
{
	if(!tn)
		return;
	PKG_NAME_(avts_foreach_recursive)(avtsn_left(tn), f, arg);	
	if(f)
		f(avtsn_value(tn), arg);
	PKG_NAME_(avts_foreach_recursive)(avtsn_right(tn), f, arg);	
}

static CTMPL_INLINE void 
PKG_NAME_(avts_foreach)(PKG_NAME_(avts) t, 
		       PKG_NAME_(avtsnode_foreach_func_t) f,
		       void *arg)
{
	ctmpl_assert(t != NULL);
	PKG_NAME_(avts_foreach_recursive)(avts_root(t), f, arg);
}

static CTMPL_INLINE void 
PKG_NAME_(avtsnode_delete)(PKG_NAME_(avts) avts,
			  PKG_NAME_(avtsnode) avtsn,
			  PKG_NAME_(avtsnode_access_func_t) df)
{
	if(!avtsn)
		return;

	PKG_NAME_(avtsnode_delete)(avts, avtsn_left(avtsn), df);
	PKG_NAME_(avtsnode_delete)(avts, avtsn_right(avtsn), df);
	if(df)		(*df)(avtsn_value(avtsn));
	CTMPL_FREE(avts->avt->free, PKG_NAME_STRING "avtnode", avtsn);
}


static void
PKG_NAME_(avts_copy_recursive)(PKG_NAME_(avtsnode) k,
			     PKG_NAME_(avts) t,
			     PKG_NAME_(avtsnode_duplicate_func_t) f)
{
	PKG_NAME_(avtsval_t) dup;	
	if(k == NULL)
		return;
	if(f){
		dup = f(avtsn_value(k));
		PKG_NAME_(avt_put)(t->avt, dup, dup);
	}
	else
		PKG_NAME_(avt_put)(t->avt, avtsn_key(k), avtsn_value(k));
	PKG_NAME_(avts_copy_recursive)(avtsn_left(k), t, f);
	PKG_NAME_(avts_copy_recursive)(avtsn_right(k), t, f);
}

static PKG_NAME_(avts_iterator) 
PKG_NAME_(avts_itr_first)(PKG_NAME_(avts) avts)
{
	PKG_NAME_(avts_iterator) ret;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = avts;
	iterator_item(ret) = PKG_NAME_(avtnode_find_min)(avts_root(avts));
	return ret;
}

static PKG_NAME_(avts_iterator) 
PKG_NAME_(avts_itr_last)(PKG_NAME_(avts) avts)
{
	PKG_NAME_(avts_iterator) ret;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = avts;
	iterator_item(ret) = PKG_NAME_(avtnode_find_max)(avts_root(avts));
	return ret;
}

static PKG_NAME_(avts_iterator) 
PKG_NAME_(avts_itr_at)(PKG_NAME_(avts) avts, iterator_pos_t idx)
{
	int prev = 0;
	PKG_NAME_(avts_iterator) ret;
	PKG_NAME_(avtsnode) avtsn;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = avts;
	
	if(idx < 0){
		++prev;
		idx = -idx - 1;
	}

	avtsn = (prev? PKG_NAME_(avtnode_find_max)(avts_root(avts)):
		     PKG_NAME_(avtnode_find_min)(avts_root(avts)));

	while(idx){
		avtsn = (prev? PKG_NAME_(avtnode_itr_prev)(avts->avt, avtsn):
			     PKG_NAME_(avtnode_itr_next)(avts->avt, avtsn));
		--idx;
	}
	iterator_item(ret) = avtsn;
	return ret;
}

static PKG_NAME_(avts_iterator) 
PKG_NAME_(avts_itr_next)(PKG_NAME_(avts_iterator) itr)
{
	PKG_NAME_(avts) t = iterator_aggregate(itr);
	PKG_NAME_(avts_iterator) ret;
	iterator_settype(ret, iterator_type(itr));
	iterator_subtype(ret) = iterator_subtype(itr);
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = NULL;

	if(iterator_is_end(itr))
		return ret;

	iterator_item(ret) = PKG_NAME_(avtnode_itr_next)
			(iterator_aggregate(ret)->avt, iterator_item(itr));

	/* multi set iterator */
	if(iterator_subtype(ret) && iterator_item(ret) &&
	   (*t->avt->cmpkey)(avtn_key(iterator_item(itr)),
			    avtn_key(iterator_item(ret))) != 0)
		iterator_item(ret) = NULL;

	return ret;
}
static PKG_NAME_(avts_iterator) 
PKG_NAME_(avts_itr_prev)(PKG_NAME_(avts_iterator) itr)
{
	PKG_NAME_(avts) t = iterator_aggregate(itr);
	PKG_NAME_(avts_iterator) ret;
	iterator_settype(ret, iterator_type(itr));
	iterator_subtype(ret) = iterator_subtype(itr);
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = NULL;

	if(iterator_is_end(itr))
		return ret;
	iterator_item(ret) = PKG_NAME_(avtnode_itr_prev)
			(iterator_aggregate(ret)->avt, iterator_item(itr));

	/* multi set iterator */
	if(iterator_subtype(ret) && iterator_item(ret) &&
	   (*t->avt->cmpkey)(avtn_key(iterator_item(itr)),
			    avtn_key(iterator_item(ret))) != 0)
		iterator_item(ret) = NULL;
	return ret;
}

static PKG_NAME_(avts_iterator) 
PKG_NAME_(avts_itr_erase_gonext)(PKG_NAME_(avts_iterator) itr)
{
	PKG_NAME_(avts) t = iterator_aggregate(itr);
	PKG_NAME_(avtsnode) n = iterator_item(itr);
	PKG_NAME_(avts_iterator) ret;
	if(iterator_is_end(itr))
		return itr;

	ret = PKG_NAME_(avts_itr_next)(itr);
	PKG_NAME_(avtnode_remove)(t->avt, n);
	CTMPL_FREE(t->avt->free, PKG_NAME_STRING "avtnode", n);
	return ret;
}

static PKG_NAME_(avts_iterator) 
PKG_NAME_(avts_itr_erase_goprev) (PKG_NAME_(avts_iterator) itr)
{
	PKG_NAME_(avts) t = iterator_aggregate(itr);
	PKG_NAME_(avtsnode) n = iterator_item(itr);
	PKG_NAME_(avts_iterator) ret;
	if(iterator_is_end(itr))
		return itr;

	ret = PKG_NAME_(avts_itr_prev)(itr);
	PKG_NAME_(avtnode_remove)(t->avt, n);
	CTMPL_FREE(t->avt->free, PKG_NAME_STRING "avtnode", n);
	return ret;
}

static int 
PKG_NAME_(avts_itr_is_equal)(PKG_NAME_(avts_iterator) x,
                            PKG_NAME_(avts_iterator) y)
{ return CTMPL_ITR_IS_EQUAL(x, y); }


static PKG_NAME_(avtsval_t) 
PKG_NAME_(avts_itr_retrieve)(PKG_NAME_(avts_iterator) itr)
{
	return avtn_key(iterator_item(itr));
}

static PKG_NAME_(avts_ctl) PKG_NAME_(avts_ctl_sw) = 
{
	{-1, -1, NULL, NULL},
	PKG_NAME_(avts_delete_by),
	PKG_NAME_(avts_clear_by),
	PKG_NAME_(avts_dup_by),
	PKG_NAME_(avts_foreach),
	PKG_NAME_(avts_count),
	PKG_NAME_(avts_itr_first),
	PKG_NAME_(avts_itr_last),
	PKG_NAME_(avts_itr_at),
	PKG_NAME_(avts_itr_next),
	PKG_NAME_(avts_itr_prev),
	PKG_NAME_(avts_itr_erase_gonext),
	PKG_NAME_(avts_itr_erase_goprev),
	PKG_NAME_(avts_itr_is_equal),
	PKG_NAME_(avts_itr_retrieve),
	PKG_NAME_(avts_get),
	PKG_NAME_(avts_put)
};

static CTMPL_INLINE void 
PKG_NAME_(avts_set_ctl)(PKG_NAME_(avts) avtsp)
{
	avtsp->ctl = &PKG_NAME_(avts_ctl_sw);
}
