/*
 * Copyright (C) 2001 and 2002 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/**
 * 	ctmpl/avltree_set.h
 *		written by Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 * 		$Id: avltree_set.h,v 1.1.1.1 2004/01/03 15:41:41 hiddy Exp $
 *	<AVL Tree Set Template>
 */

#if !defined(PKG_SETVAL_T)
#error "PKG_SETVAL_T must be defined."
#else
#define PKG_MAPKEY_T	PKG_SETVAL_T
#define PKG_MAPVAL_T	PKG_SETVAL_T
#include "avltree.h"
#undef PKG_MAPKEY_T
#undef PKG_MAPVAL_T
typedef PKG_SETVAL_T PKG_NAME_(avtsval_t);
#endif /* !defined(PKG_SETVAL_T) */

typedef struct PKG_NAME_(avts) *PKG_NAME_(avts);
typedef struct PKG_NAME_(avtnode) *PKG_NAME_(avtsnode);
typedef struct PKG_NAME_(avts_iterator) PKG_NAME_(avts_iterator);
typedef struct PKG_NAME_(avts_ctl) PKG_NAME_(avts_ctl);

/*	
 *  PKG_NAME_(avts_cmpval_func_t):
 *	this function compares two vals.
 */
typedef int (*PKG_NAME_(avts_cmpval_func_t))
	(PKG_NAME_(avtsval_t), PKG_NAME_(avtsval_t));

struct PKG_NAME_(avts){
	/* bintree */
	PKG_NAME_(avt)		avt;

	/* iterator controller */
	PKG_NAME_(avts_ctl) *ctl;
};

/* iterator for avts */
struct PKG_NAME_(avts_iterator){
	short			iter_type;  /* iterator type */
	short			iter_subtype;  /* iterator subtype */
	PKG_NAME_(avts)		iter_aggr;  /* iterator aggregate */
	PKG_NAME_(avtnode)	iter_item;  /* iterator item */
};

/*
 *  avtsnode_access_func_t
 *	this function deletes avts node. 
 */
typedef void  (*PKG_NAME_(avtsnode_access_func_t))(PKG_NAME_(avtsval_t));

/*
 *  avtsnode_foreach_func_t
 */
typedef void  (*PKG_NAME_(avtsnode_foreach_func_t))(PKG_NAME_(avtsval_t),
						  void *);

/*
 *  avtsnode_duplicate_func_t
 *	this function duplicate avts node. 
 */
typedef PKG_NAME_(avtsval_t) (*PKG_NAME_(avtsnode_duplicate_func_t))
					(PKG_NAME_(avtsval_t));


/* iterator controller for hasavtset */
struct PKG_NAME_(avts_ctl){
	PKG_NAME_(avts_iterator) enditr;
	void		         (*delete_by)(PKG_NAME_(avts), 
					PKG_NAME_(avtsnode_access_func_t));
	void		         (*clear_by)(PKG_NAME_(avts), 
					  PKG_NAME_(avtsnode_access_func_t));
	PKG_NAME_(avts)	         (*dup_by)(PKG_NAME_(avts), 
					 PKG_NAME_(avtsnode_duplicate_func_t));
	void		         (*foreach)(PKG_NAME_(avts), 
					  PKG_NAME_(avtsnode_foreach_func_t),
					  void *);
	iterator_cnt_t	         (*count)(PKG_NAME_(avts));
	PKG_NAME_(avts_iterator) (*first)(PKG_NAME_(avts));
	PKG_NAME_(avts_iterator) (*last)(PKG_NAME_(avts));
	PKG_NAME_(avts_iterator) (*at)(PKG_NAME_(avts), iterator_pos_t);
	PKG_NAME_(avts_iterator) (*next)(PKG_NAME_(avts_iterator));
	PKG_NAME_(avts_iterator) (*prev)(PKG_NAME_(avts_iterator));
	PKG_NAME_(avts_iterator) (*erase_gonext)(PKG_NAME_(avts_iterator));
	PKG_NAME_(avts_iterator) (*erase_goprev)(PKG_NAME_(avts_iterator));
	int			 (*is_equal)(PKG_NAME_(avts_iterator), 
					    PKG_NAME_(avts_iterator));
	PKG_NAME_(avtsval_t) 
			         (*retrieve)(PKG_NAME_(avts_iterator));
	PKG_NAME_(avts_iterator) (*find)(PKG_NAME_(avts), PKG_NAME_(avtsval_t));
	void		         (*insert)(PKG_NAME_(avts), PKG_NAME_(avtsval_t));
};

/* prototypes */
CTMPL_STATIC CTMPL_INLINE PKG_NAME_(avts) 
PKG_NAME_(avts_new)(PKG_NAME_(avts_cmpval_func_t));

CTMPL_STATIC CTMPL_INLINE PKG_NAME_(avts) 
PKG_NAME_(avts_new_by)(PKG_NAME_(avts_cmpval_func_t),
	 	     m_alloc_func_t, 
		     m_free_func_t);

