/*
 * Copyright (C) 2001 and 2002 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include "ctmpl/slist.c"

#ifndef _CTMPL_HASHSET_C_
#define _CTMPL_HASHSET_C_
#define hs_size(x)	((x)->size)
#define hs_count(x)	((x)->count)
#define hs_isempty(x)	(hs_count(x) == 0)
#define hs_hash(t, k)	(((*(t)->hash)(k)) % t->size)
#define hs_entry(t, k)	((t)->set[hs_hash(t, k)])

#define hsn_value(n)	(sln_data(n))

	/* !! obsolete !!*/
/* hs/int/slist/listnode/PKG_NAME_(hsnode) */
#define CTMPL_HS_FOREACH(T, I, L, LN, HN) \
	for((I) = 0; (I) < (T)->size; (I)++) \
		if(((L) = (T)->set[I]) != NULL) \
			CTMPL_SLIST_FOREACH(L, LN, (void *)HN)

#endif	/* _CTMPL_HASHSET_C_ */

static CTMPL_INLINE void PKG_NAME_(hs_set_ctl)(PKG_NAME_(hs));

/*******************************************************/
/***             External Interface                  ***/
CTMPL_STATIC CTMPL_INLINE PKG_NAME_(hs) 
PKG_NAME_(hs_new)(int s, 
 		  PKG_NAME_(hsval_hash_func_t) hk, 
		  PKG_NAME_(hsval_cmp_func_t) ck)
{
	return PKG_NAME_(hs_new_by)(s, hk, ck, NULL, NULL);
}

CTMPL_STATIC CTMPL_INLINE PKG_NAME_(hs) 
PKG_NAME_(hs_new_by)(int s, 
		     PKG_NAME_(hsval_hash_func_t) hash_func, 
		     PKG_NAME_(hsval_cmp_func_t) cmp_func,
		     m_alloc_func_t af, 
		     m_free_func_t ff)
{
	PKG_NAME_(hs) ret;
	ctmpl_assert(s >= 0);
	ctmpl_assert(hash_func != NULL);
	ctmpl_assert(cmp_func != NULL);

        /* default alloc/free function */
	if(!af)		af = PKG_NAME_(m_alloc);
	if(!ff)		ff = PKG_NAME_(m_free);

	ctmpl_assert(s > 0);
	ret = CTMPL_MALLOC(af, PKG_NAME_STRING "hs", 
				sizeof(struct PKG_NAME_(hs)));
	ret->set = CTMPL_MALLOC(af, PKG_NAME_STRING "hs->set", 
				sizeof(PKG_NAME_(slist)) * s);
	ret->size = s;
	ret->hash = hash_func;
	ret->cmp = cmp_func;
	ret->alloc = af;
	ret->free = ff;
	PKG_NAME_(hs_set_ctl)(ret);
	return ret;
}
/*******************************************************/

static CTMPL_INLINE void
PKG_NAME_(hs_put)(PKG_NAME_(hs) t, PKG_NAME_(hsval_t) v);

static CTMPL_INLINE iterator_cnt_t
PKG_NAME_(hs_count)(PKG_NAME_(hs) t) { return hs_count(t); }

static CTMPL_INLINE void
PKG_NAME_(hs_clear_by)(PKG_NAME_(hs) t,
                       PKG_NAME_(hsval_access_func_t) d)
{
	int i;
	ctmpl_assert(t);

	for(i = 0; i < hs_size(t); i++) {
		PKG_NAME_(slist) sl;

		if((sl = t->set[i]) != NULL)
			PKG_NAME_(slist_delete_by)(sl, d);
		t->set[i] = NULL;
	}
}


static CTMPL_INLINE void 
PKG_NAME_(hs_delete_by)(PKG_NAME_(hs) t, 
			PKG_NAME_(hsval_access_func_t) df)
{
	PKG_NAME_(hs_clear_by)(t, df);
	CTMPL_FREE(t->free, PKG_NAME_STRING "hs->set", t->set);
	CTMPL_FREE(t->free, PKG_NAME_STRING "hs", t);
}

static CTMPL_INLINE PKG_NAME_(hs) 
PKG_NAME_(hs_dup_by)(PKG_NAME_(hs) t, PKG_NAME_(hsval_duplicate_func_t) f)
{
	int i;
	PKG_NAME_(slist) sl;
	PKG_NAME_(listnode) ln;
	PKG_NAME_(hsval_t) hsn;
	PKG_NAME_(hs) ret;

	ctmpl_assert(t != NULL);
	ret = PKG_NAME_(hs_new_by)(t->size, t->hash, t->cmp,
				   t->alloc, t->free);

	CTMPL_HS_FOREACH(t, i, sl, ln, hsn)
		PKG_NAME_(hs_put)(ret, (f? f(hsn): hsn));

	return ret;
}


static CTMPL_INLINE void
PKG_NAME_(hs_foreach)(PKG_NAME_(hs) t,
                      PKG_NAME_(hsval_foreach_func_t) func,
		      void *arg)
{
	int i;
	ctmpl_assert(t);

	for(i = 0; i < hs_size(t); i++) {
		PKG_NAME_(slist) sl;

		if((sl = t->set[i]) != NULL)
			PKG_NAME_(slist_foreach)(sl, func, arg);
	}
}

/* hs_put returns old value if same value exists in PKG_NAME_(hs) */
static CTMPL_INLINE void
PKG_NAME_(hs_put)(PKG_NAME_(hs) t, 
		  PKG_NAME_(hsval_t) v)
{
	PKG_NAME_(slist) sl;
	PKG_NAME_(slist_iterator) itr;

	ctmpl_assert(t != NULL);
 	sl = hs_entry(t, v);

	/* no list */
	if(sl == NULL){
		hs_entry(t, v) = PKG_NAME_(slist_new_by)
				        (t->cmp, t->alloc, t->free);
		sl = hs_entry(t, v);
	}
	
	itr = iterator_find(sl, v);
	PKG_NAME_(slist_insert)(sl, v);

#ifndef PKG_USE_MULTI
	/* no same node */
	if(iterator_is_end(itr))
#endif /* PKG_USE_MULTI */
		hs_count(t)++;	/* multi map must accept new node */
}


static CTMPL_INLINE PKG_NAME_(hs_iterator)
PKG_NAME_(hs_get)(PKG_NAME_(hs) t, 
		  PKG_NAME_(hsval_t) v)
{
	PKG_NAME_(slist) sl;
	PKG_NAME_(slist_iterator) sltr;
	PKG_NAME_(hs_iterator) ret;
	ctmpl_assert(t != NULL);
	
	iterator_set_forward(ret);
#ifndef PKG_USE_MULTI
	iterator_subtype(ret) = 0;
#else 
	iterator_subtype(ret) = 1;
#endif /* PKG_USE_MULTI */
	iterator_aggregate(ret) = t;
	iterator_item(ret) = NULL;

	sl = hs_entry(t, v);
	if(sl == NULL)
		return ret;

	sltr = PKG_NAME_(slist_find)(sl, v);
	iterator_item(ret) = iterator_item(sltr);
	return ret;
}

/* 
   hs_remove removes value;
*/
static CTMPL_INLINE void
PKG_NAME_(hs_remove)(PKG_NAME_(hs) t, PKG_NAME_(hsval_t) v)
{
	PKG_NAME_(slist) sl;
	PKG_NAME_(slist_iterator) itr;
	ctmpl_assert(t != NULL);
	if((sl = hs_entry(t, v)) == NULL)
		return;

	itr = iterator_find(sl, v);
	if(iterator_is_end(itr))
		return; 

	iterator_erase(itr);
	hs_count(t)--;
	ctmpl_assert(hs_count(t) >= 0);
}

static CTMPL_INLINE PKG_NAME_(slist)
PKG_NAME_(hs_find_first_slist)(PKG_NAME_(hs) hsp)
{
	int i;
	if(hs_count(hsp) == 0)
		return NULL;

	for(i = 0; i < hs_size(hsp); i++)
		if(hsp->set[i] && sl_nodecount(hsp->set[i]))
			return hsp->set[i];

	ctmpl_assert(!"can't reach here!!");
	return NULL;
}

static CTMPL_INLINE PKG_NAME_(slist)
PKG_NAME_(hs_find_last_slist)(PKG_NAME_(hs) hsp)
{
	int i;
	if(hs_count(hsp) == 0)
		return NULL;

	for(i = hs_size(hsp) - 1; i >= 0; --i)
		if(hsp->set[i] && sl_nodecount(hsp->set[i]))
			return hsp->set[i];

	ctmpl_assert(!"can't reach here!!");
	return NULL;
}

static CTMPL_INLINE PKG_NAME_(hsnode)
PKG_NAME_(hs_find_next)(PKG_NAME_(hs) hsp, PKG_NAME_(hsnode) hsnp, int ismulti)
{
	int i = 0;
	/* no more */
	if(hsnp == NULL)
		return NULL;

	/* multi set */
	if(ismulti){
		if(sln_next(hsnp) &&
		   !(hsp->cmp(sln_data(hsnp), sln_data(sln_next(hsnp)))))
			return sln_next(hsnp);
		return NULL; /* next data is null or different value */
	}

	/* not multi set */
	if(sln_next(hsnp))
		return sln_next(hsnp);

	/* find next slist containning data */
	i = hs_hash(hsp, sln_data(hsnp));
	for(++i ; i < hs_size(hsp); ++i){
		if(hsp->set[i] && sl_nodecount(hsp->set[i]))
			return sl_headnode(hsp->set[i]);
	}
	return NULL; /* no more node */
}

static CTMPL_INLINE PKG_NAME_(hsnode)
PKG_NAME_(hs_find_prev)(PKG_NAME_(hs) hsp, PKG_NAME_(hsnode) hsnp, int ismulti)
{
	int i = 0;
	/* no more */
	if(hsnp == NULL)
		return NULL;

	/* multi set */
	if(ismulti){
		if(sln_prev(hsnp) &&
		   !(hsp->cmp(sln_data(hsnp), sln_data(sln_prev(hsnp)))))
			return sln_prev(hsnp);
		return NULL; /* prev data is null or different value */
	}

	if(sln_prev(hsnp))
		return sln_prev(hsnp);

	/* find prev slist containning data */
	i = hs_hash(hsp, sln_data(hsnp));
	for(--i ; i >= 0; --i){
		if(hsp->set[i] && sl_nodecount(hsp->set[i]))
			return sl_tailnode(hsp->set[i]);
	}
	return NULL; /* no more node */
}

static CTMPL_INLINE PKG_NAME_(hs_iterator) 
PKG_NAME_(hs_itr_first)(PKG_NAME_(hs) hsp)
{
	PKG_NAME_(slist) sl = NULL;
	PKG_NAME_(hs_iterator) ret;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = hsp;

	sl = PKG_NAME_(hs_find_first_slist)(hsp);
	if(!sl)
		iterator_item(ret) = NULL;
	else
		iterator_item(ret) = sl_headnode(sl);
	return ret;
}


static CTMPL_INLINE PKG_NAME_(hs_iterator) 
PKG_NAME_(hs_itr_last)(PKG_NAME_(hs) hsp)
{
	PKG_NAME_(slist) sl = NULL;
	PKG_NAME_(hs_iterator) ret;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = hsp;

	sl = PKG_NAME_(hs_find_last_slist)(hsp);
	if(!sl)
		iterator_item(ret) = NULL;
	else
		iterator_item(ret) = sl_tailnode(sl);
	return ret;
}

static CTMPL_INLINE PKG_NAME_(hs_iterator) 
PKG_NAME_(hs_itr_at)(PKG_NAME_(hs) hsp, iterator_pos_t idx) 
{
	int prev = 0;
	PKG_NAME_(slist) sl = NULL;
	PKG_NAME_(hsnode) node = NULL;
	PKG_NAME_(hs_iterator) ret;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = hsp;

	if(idx < 0){
		++prev;
		idx = -idx - 1;
	}

	sl = (prev? 
		PKG_NAME_(hs_find_last_slist)(hsp):
		PKG_NAME_(hs_find_first_slist)(hsp));
	if(!sl)
		iterator_item(ret) = NULL;
	else {
		node = (prev? sl_tailnode(sl): sl_headnode(sl));
		while(idx){
			node = (prev? 
				PKG_NAME_(hs_find_prev)(hsp, node, 0):
				PKG_NAME_(hs_find_next)(hsp, node, 0));
			--idx;
		}
		iterator_item(ret) = node;
	}
	return ret;
}

static CTMPL_INLINE PKG_NAME_(hs_iterator) 
PKG_NAME_(hs_itr_next)(PKG_NAME_(hs_iterator) itr)
{
	PKG_NAME_(hs_iterator) ret;
	iterator_settype(ret, iterator_type(itr));
	iterator_subtype(ret) = iterator_subtype(itr);
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = NULL;

	if(!iterator_is_end(itr)){
		iterator_item(ret) = 
			PKG_NAME_(hs_find_next)(iterator_aggregate(ret),
						iterator_item(itr),
						iterator_subtype(itr));
	}
	return ret;
}

static CTMPL_INLINE PKG_NAME_(hs_iterator) 
PKG_NAME_(hs_itr_prev)(PKG_NAME_(hs_iterator) itr)
{
	PKG_NAME_(hs_iterator) ret;
	iterator_settype(ret, iterator_type(itr));
	iterator_subtype(ret) = iterator_subtype(itr);
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = NULL;

	if(!iterator_is_end(itr)){
		iterator_item(ret) = 
			PKG_NAME_(hs_find_prev)(iterator_aggregate(ret),
						iterator_item(itr),
						iterator_subtype(itr));
	}
	return ret;
}

static CTMPL_INLINE PKG_NAME_(hs_iterator) 
PKG_NAME_(hs_itr_erase_gonext)(PKG_NAME_(hs_iterator) itr)
{
	PKG_NAME_(hs_iterator) ret;
	iterator_settype(ret, iterator_type(itr));
	iterator_subtype(ret) = iterator_subtype(itr);
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = NULL;

	if(iterator_is_end(itr))
		return ret;

	iterator_item(ret) = 
			PKG_NAME_(hs_find_next)(iterator_aggregate(ret),
						iterator_item(itr),
						iterator_subtype(itr));
	PKG_NAME_(hs_remove)(iterator_aggregate(ret),
			     sln_data(iterator_item(itr)));
	return ret;
}

static CTMPL_INLINE PKG_NAME_(hs_iterator) 
PKG_NAME_(hs_itr_erase_goprev)(PKG_NAME_(hs_iterator) itr)
{
	PKG_NAME_(hs_iterator) ret;
	iterator_settype(ret, iterator_type(itr));
	iterator_subtype(ret) = iterator_subtype(itr);
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = NULL;

	if(iterator_is_end(itr))
		return ret;

	iterator_item(ret) = 
			PKG_NAME_(hs_find_prev)(iterator_aggregate(ret),
						iterator_item(itr),
						iterator_subtype(itr));
	PKG_NAME_(hs_remove)(iterator_aggregate(ret),
			     sln_data(iterator_item(itr)));
	return ret;
}

static int 
PKG_NAME_(hs_itr_is_equal)(PKG_NAME_(hs_iterator) x,
                           PKG_NAME_(hs_iterator) y)
{ return CTMPL_ITR_IS_EQUAL(x, y); }


static CTMPL_INLINE PKG_NAME_(hsval_t)
PKG_NAME_(hs_itr_retrieve)(PKG_NAME_(hs_iterator) itr)
{
	return hsn_value(iterator_item(itr));
}

static PKG_NAME_(hs_ctl) PKG_NAME_(hs_ctl_sw) = 
{
	{-1, -1, NULL, NULL},
	PKG_NAME_(hs_delete_by),
	PKG_NAME_(hs_clear_by),
	PKG_NAME_(hs_dup_by),
	PKG_NAME_(hs_foreach),
	PKG_NAME_(hs_count),
	PKG_NAME_(hs_itr_first),
	PKG_NAME_(hs_itr_last),
	PKG_NAME_(hs_itr_at),
	PKG_NAME_(hs_itr_next),
	PKG_NAME_(hs_itr_prev),
	PKG_NAME_(hs_itr_erase_gonext),
	PKG_NAME_(hs_itr_erase_goprev),
	PKG_NAME_(hs_itr_is_equal),
	PKG_NAME_(hs_itr_retrieve),
	PKG_NAME_(hs_get),
	PKG_NAME_(hs_put)
};


static CTMPL_INLINE void 
PKG_NAME_(hs_set_ctl)(PKG_NAME_(hs) hsp)
{
	hsp->ctl = &PKG_NAME_(hs_ctl_sw);
}
