/*
 * Copyright (C) 2001 and 2002 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/*
 *	ctmpl/list.h
 *	$Id: list.h,v 1.1.1.1 2004/01/03 15:41:41 hiddy Exp $
 *		written by Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *	<General Doubly Linked List Template>
 */

#ifndef PKG_SEQVAL_T
#error "PKG_SEQVAL_T must be defined."
#else
typedef PKG_SEQVAL_T PKG_NAME_(listdata_t);
#endif /* PKG_SEQVAL_T */

#include "ctmpl/memory.h"
#include "ctmpl/ctmpl.h"

struct PKG_NAME_(list_ctl);
struct PKG_NAME_(list_iterator);
typedef struct PKG_NAME_(list) *PKG_NAME_(list);
typedef struct PKG_NAME_(listnode) *PKG_NAME_(listnode);

/*
 * 	listdata_compare_func_t:
 * 		- this function returns result of comparison between 
 *		two data of listnode.
 */
typedef int (*PKG_NAME_(listdata_compare_func_t))
	(const PKG_NAME_(listdata_t), const PKG_NAME_(listdata_t));

/*
 * 	listdata_duplicate_func_t:
 * 		- this function returns duplicate listdata
 */
typedef PKG_NAME_(listdata_t) (*PKG_NAME_(listdata_duplicate_func_t))
					(const PKG_NAME_(listdata_t));

/*
 * 	listdata_access_func_t:
 * 		- this function is used to delete data.
 */
typedef void  (*PKG_NAME_(listdata_access_func_t))(PKG_NAME_(listdata_t));

/*
 * 	listdata_foreach_func_t:
 * 		- this function is used for iterator_foreach();
 */
typedef void  (*PKG_NAME_(listdata_foreach_func_t))(PKG_NAME_(listdata_t),
						    void *);

/* iterator for list */
typedef struct PKG_NAME_(list_iterator){
	short			iter_type;  /* iterator type */
	short			iter_subtype;  /* iterator subtype */
	PKG_NAME_(list) 	iter_aggr;  /* iterator aggregate */
	PKG_NAME_(listnode) 	iter_item;  /* iterator item */
} PKG_NAME_(list_iterator);

/* iterator controller for list */
typedef struct PKG_NAME_(list_ctl){
	PKG_NAME_(list_iterator) enditr;
	void			 (*delete_by)(PKG_NAME_(list), 
				   PKG_NAME_(listdata_access_func_t));
	void			 (*clear_by)(PKG_NAME_(list), 
				   PKG_NAME_(listdata_access_func_t));
	PKG_NAME_(list)		 (*dup_by)(PKG_NAME_(list),
				   PKG_NAME_(listdata_duplicate_func_t));
	void			 (*foreach)(PKG_NAME_(list), 
				   	    PKG_NAME_(listdata_foreach_func_t),
				  	    void *);

	iterator_cnt_t		 (*count)(PKG_NAME_(list));
	PKG_NAME_(list_iterator) (*first)(PKG_NAME_(list));
	PKG_NAME_(list_iterator) (*last)(PKG_NAME_(list));
	PKG_NAME_(list_iterator) (*at)(PKG_NAME_(list), iterator_pos_t);
	PKG_NAME_(list_iterator) (*next)(PKG_NAME_(list_iterator));
	PKG_NAME_(list_iterator) (*prev)(PKG_NAME_(list_iterator));
	PKG_NAME_(list_iterator) (*erase_gonext)(PKG_NAME_(list_iterator));
	PKG_NAME_(list_iterator) (*erase_goprev)(PKG_NAME_(list_iterator));
	int 			 (*is_equal)(PKG_NAME_(list_iterator),
				   	     PKG_NAME_(list_iterator));
	PKG_NAME_(listdata_t)  	 (*retrieve)(PKG_NAME_(list_iterator));

	PKG_NAME_(list_iterator) (*find)(PKG_NAME_(list),
					 PKG_NAME_(listdata_t));
	PKG_NAME_(list_iterator) 
		(*find_between_by)(PKG_NAME_(list_iterator),
				   PKG_NAME_(list_iterator),
			   	   PKG_NAME_(listdata_t),
		      	   	   PKG_NAME_(listdata_compare_func_t));
					
	void			 (*insert_before)(PKG_NAME_(list_iterator),
					    	  PKG_NAME_(listdata_t));
	void			 (*insert_after)(PKG_NAME_(list_iterator),
					    	 PKG_NAME_(listdata_t));
	void			 (*swap)(PKG_NAME_(list_iterator),
					 PKG_NAME_(list_iterator));
	PKG_NAME_(listdata_t)	 (*assign)(PKG_NAME_(list_iterator),
					   PKG_NAME_(listdata_t));
} PKG_NAME_(list_ctl);

/* list */
struct PKG_NAME_(list) {
	/* head node of doubly linked PKG_NAME_(list) */
	PKG_NAME_(listnode) head;	

	/* tail node of doubly linked PKG_NAME_(list) */
	PKG_NAME_(listnode) tail;

	unsigned int count;	/* count of node */
	m_alloc_func_t alloc;	/* allocate function */
	m_free_func_t  free;	/* free function */

	/* iterator controller */
	PKG_NAME_(list_ctl) *ctl;
};

/* list node */
struct PKG_NAME_(listnode) { 
	PKG_NAME_(listnode) 	next;	/* next node */ 
	PKG_NAME_(listnode) 	prev;	/* prev node */	 
	PKG_NAME_(listdata_t)	data;		/* data of node */
};

/* prototypes */
CTMPL_STATIC CTMPL_INLINE PKG_NAME_(list) 
PKG_NAME_(list_new)(void);

CTMPL_STATIC CTMPL_INLINE PKG_NAME_(list) 
PKG_NAME_(list_new_by)(m_alloc_func_t, m_free_func_t);

