/*
 * Copyright (C) 2001 and 2002 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/*
 *	ctmpl/slist.c
 *	$Id: slist.c,v 1.1.1.1 2004/01/03 15:41:41 hiddy Exp $
 *	sorted doubly linked list template
 *		written by Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 */
#include "ctmpl/list.c"


#ifndef _CTMPL_SLIST_C_
#define _CTMPL_SLIST_C_
#define sl_headnode(x)	l_headnode(&(x)->_list)
#define sl_tailnode(x)	l_tailnode(&(x)->_list)
#define sl_nodecount(x)	l_nodecount(&(x)->_list)
#define sl_isempty(x)	(sl_nodecount(x) == 0)
#define sl_flags(x)		((x)->flags)
#define sln_data(x)     ln_data(x)
#define sln_next(x)     ln_next(x)
#define sln_prev(x)     ln_prev(x)

#define CTMPL_SLIST_FOREACH(L, N, V) 	\
		CTMPL_LIST_FOREACH(&((L)->_list), N, V) 
#endif  /* _CTMPL_SLIST_C_ */


/* set iterator controller */
static CTMPL_INLINE void PKG_NAME_(slist_set_ctl)(PKG_NAME_(slist));

CTMPL_STATIC CTMPL_INLINE PKG_NAME_(slist) 
PKG_NAME_(slist_new)(PKG_NAME_(listdata_compare_func_t) cf)
{
	return PKG_NAME_(slist_new_by)(cf, NULL, NULL);
}

CTMPL_STATIC CTMPL_INLINE PKG_NAME_(slist) 
PKG_NAME_(slist_new_by)(PKG_NAME_(listdata_compare_func_t) cf,
		   	m_alloc_func_t af, 
			m_free_func_t ff)
{
	PKG_NAME_(slist) ret;
	ctmpl_assert(cf != NULL);
	if(!af)		af = PKG_NAME_(m_alloc);
	if(!ff)		ff = PKG_NAME_(m_free);
	ret = CTMPL_MALLOC(af, PKG_NAME_STRING "slist", 
			   sizeof(struct PKG_NAME_(slist)));
	((PKG_NAME_(list))ret)->alloc = af;
	((PKG_NAME_(list))ret)->free = ff;
	ret->compare = cf;
	PKG_NAME_(list_set_ctl)((PKG_NAME_(list))ret);
	PKG_NAME_(slist_set_ctl)(ret);
	return ret;
}

static CTMPL_INLINE void
PKG_NAME_(slist_clear_by)(PKG_NAME_(slist) sl,  
                          PKG_NAME_(listdata_access_func_t) d)
{
	PKG_NAME_(list_clear_by)((PKG_NAME_(list))sl, d);
}


static CTMPL_INLINE void 
PKG_NAME_(slist_delete_by)(PKG_NAME_(slist) s, 
			   PKG_NAME_(listdata_access_func_t) df)
{
	PKG_NAME_(slist_clear_by)(s, df);
	CTMPL_FREE(((PKG_NAME_(list))s)->free, PKG_NAME_STRING "slist", s);
}

static CTMPL_INLINE PKG_NAME_(slist) 
PKG_NAME_(slist_dup_by)(PKG_NAME_(slist) s, 
			PKG_NAME_(listdata_duplicate_func_t) f)
{
	PKG_NAME_(slist) ret;
	PKG_NAME_(list)  lp;
	PKG_NAME_(listnode) ln;
	PKG_NAME_(listdata_t) val;

	lp = (PKG_NAME_(list))s;

	ret = PKG_NAME_(slist_new_by)(s->compare, lp->alloc, lp->free);

	lp = (PKG_NAME_(list))ret;
	CTMPL_SLIST_FOREACH(s, ln, val)
		iterator_append(lp, (f? f(val): val));
	return ret;
}
static void
PKG_NAME_(slist_foreach)(PKG_NAME_(slist) l, 
			PKG_NAME_(listdata_foreach_func_t) func, 
			void *arg)
{
	iterator_foreach(&l->_list, func, arg);
}

static iterator_cnt_t  
PKG_NAME_(slist_count)(PKG_NAME_(slist) lst)
{
	return sl_nodecount(lst);
}

/* binary search code */
static CTMPL_INLINE PKG_NAME_(slistnode) 
PKG_NAME_(slistnode_go)(PKG_NAME_(slistnode) n,  
		        int i)
{
	register int x = (i < 0)? -i: i; 
	int isminus = (i < 0);
	while(x > 0 && n){
		n = (isminus? sln_prev(n): sln_next(n));
		--x;
	}
	return n;
}

static CTMPL_INLINE void
PKG_NAME_(slist_insert)(PKG_NAME_(slist) s, 
		     	PKG_NAME_(slistval_t) v)
{
	int cmp = 0;
	int pos, mid, left, right, last;
	PKG_NAME_(list) lp;
	register PKG_NAME_(listnode) n1;

	ctmpl_assert(s != NULL);

	lp = (PKG_NAME_(list))s;

	if(sl_isempty(s) ||
	  (*s->compare)(v, sln_data(sl_headnode(s))) < 0){
		iterator_prepend(lp, v);
		return;
	}
	else if((*s->compare)(v, sln_data(sl_tailnode(s))) > 0){
		iterator_append(lp, v);
		return;
	}

	left = 0;
	last = right = sl_nodecount(s) - 1;
	n1 = sl_headnode(s);
	pos = 0;
	
	while(left < right){
		mid = (left + right) / 2;
		n1 = PKG_NAME_(slistnode_go)(n1, mid - pos);
		pos = mid;

		cmp = s->compare(sln_data(n1), v);
		if(cmp < 0)
			left = mid + 1;
		else 
			right = mid;
	}
	n1 = PKG_NAME_(slistnode_go)(n1, left - pos);
	cmp = s->compare(sln_data(n1), v);

#ifndef PKG_USE_MULTI	/* not use multi value */
	if(!cmp) /* same data entry */
			sln_data(n1) = v;
	else {
#endif /* PKG_USE_MULTI */

		if(cmp > 0)
			n1 = ln_prev(n1);
		PKG_NAME_(listnode_insert)(lp, n1, v);

#ifndef PKG_USE_MULTI	/* not use multi value */
	}
#endif /* PKG_USE_MULTI */
}
	/* binary search */
static CTMPL_INLINE PKG_NAME_(slist_iterator) 
PKG_NAME_(slist_find)(PKG_NAME_(slist) s,  
		      const PKG_NAME_(slistval_t) v) {
	int cmp;
	int pos, mid, left, right, last;
	PKG_NAME_(slistnode) n1;
	PKG_NAME_(slist_iterator) ret;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
#ifndef PKG_USE_MULTI	/* not use multi value */
	iterator_subtype(ret) = 0;
#else 
	iterator_subtype(ret) = 1;
#endif /* PKG_USE_MULTI	*/
	iterator_aggregate(ret) = s;
	iterator_item(ret) = NULL;

	ctmpl_assert(s != NULL);
	if(sl_isempty(s))
		return ret;

	left = 0;
	last = right = sl_nodecount(s) - 1;
	n1 = sl_headnode(s);
	pos = 0;
	
	while(left < right){
		mid = (left + right) / 2;
		n1 = PKG_NAME_(slistnode_go)(n1, mid - pos);
		pos = mid;

		cmp = s->compare(sln_data(n1), v);
		if(cmp < 0)
			left = mid + 1;
		else 
			right = mid;
	}
	n1 = PKG_NAME_(slistnode_go)(n1, left - pos);
	cmp = s->compare(sln_data(n1), v);
	if(!cmp)
		iterator_item(ret) = n1;
	return ret;
}

static CTMPL_INLINE PKG_NAME_(slist_iterator) 
PKG_NAME_(slist_itr_first)(PKG_NAME_(slist) slp)
{
	PKG_NAME_(slist_iterator) ret;
	iterator_set_forward(ret);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = slp;
	ctmpl_assert(slp);
	iterator_item(ret) = sl_headnode(slp);
	return ret;
}

static CTMPL_INLINE PKG_NAME_(slist_iterator) 
PKG_NAME_(slist_itr_last)(PKG_NAME_(slist) slp)
{
	PKG_NAME_(slist_iterator) ret;
	iterator_set_forward(ret);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = slp;
	ctmpl_assert(slp);
	iterator_item(ret) = sl_tailnode(slp);
	return ret;
}

static CTMPL_INLINE PKG_NAME_(slist_iterator) 
PKG_NAME_(slist_itr_at)(PKG_NAME_(slist) slp, iterator_pos_t pos)
{
	PKG_NAME_(slist_iterator) ret;
	PKG_NAME_(list_iterator) itr;
	iterator_set_forward(ret);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = slp;
	itr = iterator_at(&slp->_list, pos);

	iterator_item(ret) = iterator_item(itr); 
	return ret;
}

static CTMPL_INLINE PKG_NAME_(slist_iterator) 
PKG_NAME_(slist_itr_next)(PKG_NAME_(slist_iterator) itr)
{
	PKG_NAME_(slist) slp;
	PKG_NAME_(slist_iterator) ret;
	iterator_settype(ret, iterator_type(itr));
	iterator_subtype(ret) = iterator_subtype(itr);
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = NULL;

	if(iterator_is_end(itr))
		return itr;

	/* if multi value is enabled, compare next value */
	if(iterator_subtype(ret)){
		slp = iterator_aggregate(itr);
		if(sln_next(iterator_item(itr)) && /* next is not NULL */
		   !slp->compare(sln_data(iterator_item(itr)), 
				 sln_data(sln_next(iterator_item(itr)))))
			iterator_item(ret) = sln_next(iterator_item(itr));
	}
	else 
		iterator_item(ret) = sln_next(iterator_item(itr));
	return ret;
}

static CTMPL_INLINE PKG_NAME_(slist_iterator) 
PKG_NAME_(slist_itr_prev)(PKG_NAME_(slist_iterator) itr)
{
	PKG_NAME_(slist) slp;
	PKG_NAME_(slist_iterator) ret;
	iterator_settype(ret, iterator_type(itr));
	iterator_subtype(ret) = iterator_subtype(itr);
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = NULL;

	if(iterator_is_end(itr))
		return itr;

	/* if multi value is enabled, compare prev value */
	if(iterator_subtype(ret)){
		slp = (PKG_NAME_(slist))iterator_aggregate(itr);
		if(sln_prev(iterator_item(itr)) && /* prev is not NULL */
		   !slp->compare(sln_data(iterator_item(itr)), 
				 sln_data(sln_prev(iterator_item(itr)))))
			iterator_item(ret) = sln_prev(iterator_item(itr));
	}
	else 
		iterator_item(ret) = sln_prev(iterator_item(itr));
	return ret;
}

static CTMPL_INLINE PKG_NAME_(slist_iterator) 
PKG_NAME_(slist_itr_erase_gonext)(PKG_NAME_(slist_iterator) itr)
{
	PKG_NAME_(slist_iterator) ret;

	/* no item to erase */
	if(iterator_is_end(itr))
		return itr;

	ret = PKG_NAME_(slist_itr_next)(itr);
	PKG_NAME_(listnode_remove)((PKG_NAME_(list))iterator_aggregate(itr),
						 iterator_item(itr));
	return ret;
}

static CTMPL_INLINE PKG_NAME_(slist_iterator) 
PKG_NAME_(slist_itr_erase_goprev)(PKG_NAME_(slist_iterator) itr)
{
	PKG_NAME_(slist_iterator) ret;

	/* no item to erase */
	if(iterator_is_end(itr))
		return itr;

	ret = PKG_NAME_(slist_itr_next)(itr);
	PKG_NAME_(listnode_remove)((PKG_NAME_(list))iterator_aggregate(itr),
						  iterator_item(itr));
	return ret;
}

static int 
PKG_NAME_(slist_itr_is_equal)(PKG_NAME_(slist_iterator) x,
                              PKG_NAME_(slist_iterator) y)
{ return CTMPL_ITR_IS_EQUAL(x, y); }


static CTMPL_INLINE PKG_NAME_(listdata_t)
PKG_NAME_(slist_itr_retrieve)(PKG_NAME_(slist_iterator) itr)
{
	return sln_data(iterator_item(itr));
}


static PKG_NAME_(slist_ctl) PKG_NAME_(slist_ctl_sw) = 
{
	{ -1, -1, NULL, NULL},
	PKG_NAME_(slist_delete_by),
	PKG_NAME_(slist_clear_by),
	PKG_NAME_(slist_dup_by),
	PKG_NAME_(slist_foreach),
	PKG_NAME_(slist_count),
	PKG_NAME_(slist_itr_first),
	PKG_NAME_(slist_itr_last),
	PKG_NAME_(slist_itr_at),
	PKG_NAME_(slist_itr_next),
	PKG_NAME_(slist_itr_prev),
	PKG_NAME_(slist_itr_erase_gonext),
	PKG_NAME_(slist_itr_erase_goprev),
	PKG_NAME_(slist_itr_is_equal),
	PKG_NAME_(slist_itr_retrieve),
	PKG_NAME_(slist_find),
	PKG_NAME_(slist_insert)
};

static CTMPL_INLINE void 
PKG_NAME_(slist_set_ctl)(PKG_NAME_(slist) slp)
{
	slp->ctl = &PKG_NAME_(slist_ctl_sw);
}
