/*
 * Copyright (C) 2001 and 2002 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/**
 * 	ctmpl/vector.h
 *		written by Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 * 		$Id: vector.h,v 1.1.1.1 2004/01/03 15:41:41 hiddy Exp $
 */

#ifndef _CTMPL_VECTOR_H_
#define _CTMPL_VECTOR_H_

/* default vector initialized size */
#define v_size(x)		(x->size)
#define v_getAt(x, y)		(x->array[y])
#endif /*_CTMPL_VECTOR_H_*/

#include "ctmpl/ctmpl.h"
#include "ctmpl/memory.h"

#ifndef PKG_SEQVAL_T	
#error  "PKG_SEQVAL_T must be defined."
#endif

typedef struct PKG_NAME_(vector) *PKG_NAME_(vector);
typedef PKG_SEQVAL_T  PKG_NAME_(vecdata_t);
typedef struct PKG_NAME_(vector_iterator) PKG_NAME_(vector_iterator);
typedef struct PKG_NAME_(vector_ctl) PKG_NAME_(vector_ctl);

typedef int (*PKG_NAME_(vecdata_compare_func_t))(PKG_NAME_(vecdata_t), 
						 PKG_NAME_(vecdata_t));
typedef void (*PKG_NAME_(vecdata_access_func_t))(PKG_NAME_(vecdata_t));
typedef PKG_NAME_(vecdata_t) 
	(*PKG_NAME_(vecdata_duplicate_func_t))(PKG_NAME_(vecdata_t));

typedef void (*PKG_NAME_(vecdata_foreach_func_t))(PKG_NAME_(vecdata_t),
						  void *);

struct PKG_NAME_(vector){
	unsigned int size;		/* size */
	unsigned int capacity;		/* capacity of array */
	unsigned int expandsize;	/* expand size */
	PKG_NAME_(vecdata_t) *array;
	m_alloc_func_t alloc;   /* allocate function */
	m_free_func_t  free;    /* free function */
	PKG_NAME_(vector_ctl) *ctl;
};

struct PKG_NAME_(vector_iterator){
	short		     	iter_type;  /* iterator type */
	short		     	iter_subtype;  /* iterator subtype */
	PKG_NAME_(vector)   	iter_aggr;  /* iterator aggregate */
	int			iter_item;  /* iterator item */
};

struct PKG_NAME_(vector_ctl){
	PKG_NAME_(vector_iterator)  	enditr;
	void			   (*delete_by)(PKG_NAME_(vector), 
				   PKG_NAME_(vecdata_access_func_t));
	void			   (*clear_by)(PKG_NAME_(vector), 
				   PKG_NAME_(vecdata_access_func_t));
	PKG_NAME_(vector)		 (*dup_by)(PKG_NAME_(vector),
				   PKG_NAME_(vecdata_duplicate_func_t));
	void			   (*foreach)(PKG_NAME_(vector), 
				   PKG_NAME_(vecdata_foreach_func_t),
				   void *);

	iterator_cnt_t		   (*count)(PKG_NAME_(vector));
	PKG_NAME_(vector_iterator) (*first)(PKG_NAME_(vector));
	PKG_NAME_(vector_iterator) (*last)(PKG_NAME_(vector));
	PKG_NAME_(vector_iterator) (*at)(PKG_NAME_(vector), iterator_pos_t);
	PKG_NAME_(vector_iterator) (*next)(PKG_NAME_(vector_iterator));
	PKG_NAME_(vector_iterator) (*prev)(PKG_NAME_(vector_iterator));
	PKG_NAME_(vector_iterator) (*erase_gonext)(PKG_NAME_(vector_iterator));
	PKG_NAME_(vector_iterator) (*erase_goprev)(PKG_NAME_(vector_iterator));
	int			   (*is_equal)(PKG_NAME_(vector_iterator),
					       PKG_NAME_(vector_iterator));
	PKG_NAME_(vecdata_t)  	   (*retrieve)(PKG_NAME_(vector_iterator));

	PKG_NAME_(vector_iterator) (*find)(PKG_NAME_(vector),
					 PKG_NAME_(vecdata_t));
	PKG_NAME_(vector_iterator) 
		(*find_between_by)(PKG_NAME_(vector_iterator),
				   PKG_NAME_(vector_iterator),
			   	   PKG_NAME_(vecdata_t),
		      	   	   PKG_NAME_(vecdata_compare_func_t));
					
	void			   (*insert_before)(PKG_NAME_(vector_iterator),
					    	  PKG_NAME_(vecdata_t));
	void			   (*insert_after)(PKG_NAME_(vector_iterator),
					    	 PKG_NAME_(vecdata_t));
	void			   (*swap)(PKG_NAME_(vector_iterator),
					   PKG_NAME_(vector_iterator));
	PKG_NAME_(vecdata_t)       (*assign)(PKG_NAME_(vector_iterator),
					     PKG_NAME_(vecdata_t));
}; 

/*   initialized vector size & expand size */
CTMPL_STATIC CTMPL_INLINE PKG_NAME_(vector) 
PKG_NAME_(vector_new)(int, 	/* initialized vector size */
		      int);	/* expand size */

CTMPL_STATIC CTMPL_INLINE PKG_NAME_(vector) 
PKG_NAME_(vector_new_by)(int, int, m_alloc_func_t, m_free_func_t);

