/*
 *			dictbook.c
 * $Id: dictbook.c,v 1.7 2004/01/11 21:00:59 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <stdio.h>
#include <string.h>
#include "memory.h"
#include "bitmap.h"
#include "dictbook.h"

int                     
dict_open(struct dictbook *dbp)
{
	if(DBOOK_IS_OPEN(dbp))
		return 1;
	dbp->db_flags |= DBFLAG_OPEN;
	return dbp->db_open(dbp);
}

int
dict_close(struct dictbook *dbp)
{
	if(!DBOOK_IS_OPEN(dbp))
		return 1;
	dbp->db_flags &= ~DBFLAG_OPEN;
	return dbp->db_close(dbp);
}

void	
dict_free(struct dictbook *dbp)
{
	if(DBOOK_IS_OPEN(dbp))
		dbp->db_close(dbp);
	dbp->db_free(dbp);
}

#define	CHECK_FLAGS(x)		do {  			\
		if(!DBOOK_IS_OPEN(x)){ 			\
			dbp->db_errno = EDBNOTOPENED;	\
			return 0;			\
		}					\
		else if(!DBOOK_IS_AVAILABLE(x)){ 	\
			dbp->db_errno = EDBNOTAVAIL;	\
			return 0;			\
		}					\
	} while(0)

int	
dict_reclaim(struct dictbook *dbp) 
{ 
	CHECK_FLAGS(dbp);
	dbp->db_reclaim(dbp); 
	return 1;
}

int	
dict_error(struct dictbook *dbp)
{ 
	return dbp->db_errno; 
}

const char *
dict_strerr(struct dictbook *dbp){ 
	if(dbp->db_errno == EDBNOTOPENED)
		return "Book is not opend.";
	if(dbp->db_errno == EDBNOTAVAIL)
		return "Book is not avalable.";
	return dbp->db_strerr(dbp); 
}

/* caller must free returned pointers */
strlist	
dict_lookup(struct dictbook *dbp, const char *word)
{ 
	strlist slp;
	CHECK_FLAGS(dbp);
	if((slp = dbp->db_lookup(dbp, word)) == NULL)
		return NULL;
	return strlist_dup(slp);
}

strlist	dict_getentry(struct dictbook *dbp, const char *word)
{ 
	strlist slp;
	CHECK_FLAGS(dbp);
	if((slp = dbp->db_getentry(dbp, word)) == NULL)
		return NULL;
	return strlist_dup(slp);
}

struct bitmap *
dict_getbitmap(struct dictbook *dbp, const char *fontid, 
	       unsigned int width,  unsigned int height)
{
	int i = 0;
	int w;
	int bestsize = -1;
	struct bitmap *p;

	CHECK_FLAGS(dbp);
	/* making decision which size of font should be used */
	for(i = 0; i < dbp->db_nfontsize; ++i){
		if(height == dbp->db_fontsizelist[i]){
			bestsize = i;
			break;
		}

		if(bestsize == -1){
			bestsize = i;
			continue;
		}

		if(dbp->db_fontsizelist[i] > height){
			if(dbp->db_fontsizelist[bestsize] < height)
				bestsize = i;
			else if(dbp->db_fontsizelist[i] < dbp->db_fontsizelist[bestsize])
				bestsize = i;  /* closer */
		}
		else if(dbp->db_fontsizelist[i] > dbp->db_fontsizelist[bestsize])
				bestsize = i;  /* closer */
	}

	if(bestsize == -1)
		return NULL;

	p = dbp->db_getbitmap(dbp, fontid, bestsize);

	if(!p)
		return NULL;

	w = width;
	if(!width)
		w = (int)((double)p->bm_width * 
			  (double)height/(double)p->bm_height);

	return bitmap_scale(p, w, height);
}
