/*
 *                      resource.c
 * $Id: resource.c,v 1.2 2004/01/22 21:08:59 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <stdio.h>
#include "memory.h"
#include "bitmap.h"
#include "xwindow.h"
#include "resource.h"

/* cursor bitmaps */
#include "bitmaps/handclick.bmp"
#include "bitmaps/handclickmask.bmp"
#include "bitmaps/handgrab.bmp"
#include "bitmaps/handgrabmask.bmp"
#include "bitmaps/handopen.bmp"
#include "bitmaps/handopenmask.bmp"
#include "bitmaps/handptr.bmp"
#include "bitmaps/handptrmask.bmp"

static struct bitmap handptr_bmp = { handptr_width, handptr_height, 
                                     handptr_x_hot, handptr_y_hot, 
                                     handptr_bits };
static struct bitmap handgrab_bmp = { handgrab_width, handgrab_height, 
                                     handgrab_x_hot, handgrab_y_hot, 
                                     handgrab_bits };
static struct bitmap handopen_bmp = { handopen_width, handopen_height, 
                                     handopen_x_hot, handopen_y_hot, 
                                     handopen_bits };
static struct bitmap handclick_bmp = { handclick_width, handclick_height, 
                                     handclick_x_hot, handclick_y_hot, 
                                     handclick_bits };
static struct bitmap handptrmask_bmp = { handptrmask_width, handptrmask_height, 
                                        -1, -1, handptrmask_bits };
static struct bitmap handgrabmask_bmp = { handgrabmask_width, handgrabmask_height, 
                                        -1, -1, handgrabmask_bits };
static struct bitmap handopenmask_bmp = { handopenmask_width, handopenmask_height, 
                                        -1, -1, handopenmask_bits };
static struct bitmap handclickmask_bmp = { handclickmask_width, handclickmask_height, 
                                        -1, -1, handclickmask_bits };

/* static resource */
static struct stk_resource stkr;

void 
stk_initialize_resource(xrootwindow *xwp)
{
	/* create cursors */
	stkr.xc_handptr = xcursor_new(xwp, &handptr_bmp, &handptrmask_bmp, 
				      handptr_bmp.bm_hotx, handptr_bmp.bm_hoty, 
				      0xffffff, 0x0);
	stkr.xc_handgrab = xcursor_new(xwp, &handgrab_bmp, &handgrabmask_bmp, 
				      handgrab_bmp.bm_hotx, handgrab_bmp.bm_hoty, 
				      0xffffff, 0x0);
	stkr.xc_handopen = xcursor_new(xwp, &handopen_bmp, &handopenmask_bmp, 
				      handopen_bmp.bm_hotx, handopen_bmp.bm_hoty, 
				      0xffffff, 0x0);
	stkr.xc_handclick = xcursor_new(xwp, &handclick_bmp, &handclickmask_bmp, 
				     handclick_bmp.bm_hotx, handclick_bmp.bm_hoty, 
				      0xffffff, 0x0);
}

void stk_finalize_resource(xrootwindow *xrwp)
{
	xcursor_free(xrwp, stkr.xc_handptr);
	xcursor_free(xrwp, stkr.xc_handgrab);
	xcursor_free(xrwp, stkr.xc_handclick);
}

struct 
stk_resource *stk_get_resource(void)
{
	return &stkr;
}

