/*
 *                      stkbase.h
 * $Id: stkbase.c,v 1.11 2004/01/23 02:45:27 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <stdio.h>
#include "bitmap.h"
#include "xwindow.h"
#include "memory.h"
#include "resource.h"
#include "xobject.h"
#include "xsignal.h"
#include "xactor.h"

typedef void *(*stk_malloc_t)(const char *, size_t, const char *, int);
typedef void (*stk_free_t)(const char *, void *, const char *, int);

void 
stk_initialize(xrootwindow *xrwp, stk_malloc_t mp, stk_free_t fp)
{
	/* must be pair */
	if(mp && fp){
		stk_malloc = mp;	
		stk_free = fp;	
	}
	stk_initialize_resource(xrwp);
}

void stk_finalize(xrootwindow *xrwp)
{
	stk_finalize_resource(xrwp);
}


/*!!!!!!!!!!   these code is written as just temporary code for xpopdic !!!!! */ 
static int stk_stop = 0;
static xwindow *stkxwp;
static struct xobject *stktop;
void stk_stop_loop(void){ stk_stop = 1; }

/* event handlers and structure for xpopdic */
static int stk_button_pressed(xwindow *, struct xw_event_handler *, int x, int y, int button);
static int stk_button_released(xwindow *, struct xw_event_handler *, int x, int y, int button)
;
static int stk_button_motion(xwindow *, struct xw_event_handler *, int x, int y, int button);
static int stk_key_pressed(xwindow *, struct xw_event_handler *, int x, int y, const char *keyname, int modifier);
static int stk_key_released(xwindow *, struct xw_event_handler *, int x, int y, const char *keyname, int modifier);
static int stk_expose(xwindow *, struct xw_event_handler *);
static int stk_noexpose(xwindow *, struct xw_event_handler *);
static int stk_visibility_changed(xwindow *, struct xw_event_handler *, int notify_type); 
static int stk_mouse_in(xwindow *, struct xw_event_handler *, int x, int y);
static int stk_mouse_out(xwindow *, struct xw_event_handler *, int x, int y);
static int stk_focus_in(xwindow *, struct xw_event_handler *);
static int stk_focus_out(xwindow *, struct xw_event_handler *);

struct xw_event_handler stk_eh = 
        { stk_button_pressed, stk_button_released, stk_button_motion, 
          stk_key_pressed, stk_key_released, stk_expose, stk_noexpose, 
          stk_visibility_changed, stk_mouse_in, stk_mouse_out, 
          stk_focus_in, stk_focus_out };


void stk_start_loop(void (*before)(void), void (*after)(void))
{
	if(stkxwp == NULL || stktop == NULL)
		return;
	stk_stop = 0;
	xw_event_loop(stkxwp, &stk_eh, before, after);
}

void stk_map(void)
{
	if(stkxwp)
		xw_map(stkxwp);
}
void stk_unmap(void)
{
	if(stkxwp)
		xw_unmap(stkxwp);
}
void stk_draw(void)
{
	if(stkxwp && stktop)
		xobj_draw(stktop, 0, 0, stkxwp, 0);
}
void stk_redraw(void)
{
	if(stkxwp && stktop)
		xobj_redraw(stktop, 0, 0, stkxwp, 0);
}

void stk_set(xwindow *xwp, struct xobject *p)
{
	/* send XSIG_PROXY_CACHERESET */
	struct xsigarg sarg = { 0, 0 };
	int sig = XSIG_PROXY_CACHERESET; 
	int flags = xcmpfind_build_flags(XCMPFIND_FIRSTMATCH, XOBJTYPE_ANY, sig);
	struct xobject *o;
	xpoint pos = {0, 0};

	/* send PROXY_CACHERESET signal to current object */
	if(stktop){
		o = xcmp_find(stktop, 0, 0, flags, &pos);
		if(o && xobj_is_actor(o)){
			struct xactor *a = (struct xactor *)o;
			xact_send_signal(a, pos, sig, (struct xsigarg *)&sarg, stktop, stkxwp);
		}
	}

	stkxwp = xwp; 
	stktop = p; 

	if(p){
		xcmp_compose(p);
		xcmp_relocate(p, 0, 0);
	}
}

static int
stk_button_pressed(xwindow *xwp, struct xw_event_handler *ehp, int x, int y, int button)
{
	/* send XSIG_BTNPRESS */
	struct xs_btninfo sarg = { {x, y}, button };
	int sig = XSIG_BTNPRESS;
	int flags = xcmpfind_build_flags(XCMPFIND_FIRSTMATCH, XOBJTYPE_ANY, sig);
	struct xobject *o;
	xpoint pos = {0, 0};

	o = xcmp_find(stktop, 0, 0, flags, &pos);

	if(o && xobj_is_actor(o)){
		struct xactor *a = (struct xactor *)o;
		xact_send_signal(a, pos, sig, (struct xsigarg *)&sarg, stktop, xwp);
	}

	xroot_flush(xwp->xw_root);
	return (stk_stop == 0);
}

static int
stk_button_released(xwindow *xwp, struct xw_event_handler *ehp, int x, int y, int button)
{
	/* send XSIG_BTNRELEASE */
	struct xs_btninfo sarg = { {x, y}, button };
	int sig = XSIG_BTNRELEASE;
	int flags = xcmpfind_build_flags(XCMPFIND_FIRSTMATCH, XOBJTYPE_ANY, sig);
	struct xobject *o;
	xpoint pos = {0, 0};

	o = xcmp_find(stktop, 0, 0, flags, &pos);

	if(o && xobj_is_actor(o)){
		struct xactor *a = (struct xactor *)o;
		xact_send_signal(a, pos, sig, (struct xsigarg *)&sarg, stktop, xwp);
	}

	xroot_flush(xwp->xw_root);
	return (stk_stop == 0);
}

static int
stk_button_motion(xwindow *xwp, struct xw_event_handler *ehp, int x, int y, int button)
{
	/* send XSIG_MOTION */
	struct xs_bmotion sarg = { {x, y}, button };
	int sig = XSIG_MOTION;
	int flags = xcmpfind_build_flags(XCMPFIND_FIRSTMATCH, XOBJTYPE_ANY, sig);
	struct xobject *o;
	xpoint pos = {0, 0};

	o = xcmp_find(stktop, 0, 0, flags, &pos);

	if(o && xobj_is_actor(o)){
		struct xactor *a = (struct xactor *)o;
		xact_send_signal(a, pos, sig, (struct xsigarg *)&sarg, stktop, xwp);
	}

	xroot_flush(xwp->xw_root);
	return (stk_stop == 0);
}

static int
stk_key_pressed(xwindow *xwp, struct xw_event_handler *ehp, 
		int x, int y, const char *keyname, int modifier)
{
	/* send XSIG_KEYPRESS */
	struct xs_keyinfo sarg = { {x, y}, keyname, modifier };
	int sig = XSIG_KEYPRESS;
	int flags = xcmpfind_build_flags(XCMPFIND_FIRSTMATCH, XOBJTYPE_ANY, sig);
	struct xobject *o;
	xpoint pos = {0, 0};

	o = xcmp_find(stktop, 0, 0, flags, &pos);

	if(o && xobj_is_actor(o)){
		struct xactor *a = (struct xactor *)o;
		xact_send_signal(a, pos, sig, (struct xsigarg *)&sarg, stktop, xwp);
	}

	xroot_flush(xwp->xw_root);
	return (stk_stop == 0);
}

static int
stk_key_released(xwindow *xwp, struct xw_event_handler *ehp, int x, int y, const char *keyname, int modifier)
{
	/* send XSIG_KEYRELEASE */
	struct xs_keyinfo sarg = { {x, y}, keyname, modifier };
	int sig = XSIG_KEYRELEASE;
	int flags = xcmpfind_build_flags(XCMPFIND_FIRSTMATCH, XOBJTYPE_ANY, sig);
	struct xobject *o;
	xpoint pos = {0, 0};

	o = xcmp_find(stktop, 0, 0, flags, &pos);

	if(o && xobj_is_actor(o)){
		struct xactor *a = (struct xactor *)o;
		xact_send_signal(a, pos, sig, (struct xsigarg *)&sarg, stktop, xwp);
	}

	xroot_flush(xwp->xw_root);
	return (stk_stop == 0);
}

static int
stk_expose(xwindow *xwp, struct xw_event_handler *ehp)
{
	/* redraw */
	xobj_draw(stktop, 0, 0, xwp, 0);
	xroot_flush(xwp->xw_root);
	return (stk_stop == 0);
}

static int
stk_noexpose(xwindow *xwp, struct xw_event_handler *ehp)
{
	/* do nothing */
	return (stk_stop == 0);
}

static int
stk_visibility_changed(xwindow *xwp, struct xw_event_handler *ehp, int notify_type) 
{
	/* do nothing */
	/* ignore */
	if(notify_type != 0){
		xw_raise(xwp);
		xroot_flush(xwp->xw_root);
	}

	return (stk_stop == 0);
}

static int
stk_mouse_in(xwindow *xwp, struct xw_event_handler *ehp, int x, int y)
{
	/* send XSIG_MOTION */
	struct xsigarg sarg = {-1, -1};
	int flags = xcmpfind_build_flags(XCMPFIND_FIRSTMATCH, XOBJTYPE_ANY, XSIG_MOTION);
	struct xobject *o;
	xpoint pos = {0, 0};

	o = xcmp_find(stktop, x, y, flags, &pos);

	if(o && xobj_is_actor(o)){
		struct xactor *a = (struct xactor *)o;
		xact_send_signal(a, pos, XSIG_LEAVE, &sarg, stktop, xwp);
	}
	return (stk_stop == 0);
}

static int
stk_mouse_out(xwindow *xwp, struct xw_event_handler *ehp, int x, int y)
{
#if 0
	/* send XSIG_LEAVE */
	struct xsigarg sarg = {-1, -1};
	int flags = xcmpfind_build_flags(XCMPFIND_FIRSTMATCH, XOBJTYPE_ANY, XSIG_LEAVE);
	struct xobject *o;
	xpoint pos = {0, 0};

	o = xcmp_find(stktop, 0, 0, flags, &pos);

	if(o && xobj_is_actor(o)){
		struct xactor *a = (struct xactor *)o;
		xact_send_signal(a, pos, XSIG_LEAVE, &sarg, stktop, xwp);
	}
#endif
	return (stk_stop == 0);
}

static int
stk_focus_in(xwindow *xwp, struct xw_event_handler *ehp)
{
	/* do nothing */
	return (stk_stop == 0);
}

static int
stk_focus_out(xwindow *xwp, struct xw_event_handler *ehp)
{
	/* do nothing */
	return (stk_stop == 0);
}

