/*
 *			wordhist.c
 * $Id: wordhist.c,v 1.7 2004/01/07 13:37:28 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include "memory.h"
#include "wordhist.h"

#include "whmap.c"
whmap _whmap;	/* map for word history */ 

static int 
whmap_cmp(const char *p1, const char *p2)
{ 
	return strcmp(p1, p2); 
}

static unsigned int 
whmap_hash(const char *p1)
{ 
	int i;
	unsigned int ret = 0;

	for(i = 0; i < 4; ++i){
		if(p1[i] == '\0')
			break;
		ret += p1[i];
	}
	return ret;
}

static void
mkdatestr(struct whentry *whe)
{
	char buf[64];
	struct tm *tmp;

	if(whe->whe_count == 1)
		snprintf(buf, 64, "1st lookup");
	else {
		if(whe->whe_ldatestr)
			xp_strfree(whe->whe_ldatestr);
		tmp = localtime(&whe->whe_lasttime);
		switch(whe->whe_count){
		case 2:
			snprintf(buf, 64, "#2(last:%d/%02d/%02d-%02d %02d)", tmp->tm_year + 1900, 
			tmp->tm_mon+1, tmp->tm_mday, tmp->tm_hour, tmp->tm_min);
			break;
		case 3:
			snprintf(buf, 64, "#3(last:%d/%02d/%02d %02d:%02d)", tmp->tm_year + 1900, 
			tmp->tm_mon+1, tmp->tm_mday, tmp->tm_hour, tmp->tm_min);
			break;
		default:
			snprintf(buf, 64, "#%d(last:%d/%02d/%02d %02d:%02d)", whe->whe_count,
			tmp->tm_year + 1900, tmp->tm_mon+1, tmp->tm_mday, tmp->tm_hour, tmp->tm_min);
			break;
		}
	}
	whe->whe_ldatestr = xp_strdup(buf);	
}

struct whentry *
whentry_alloc(const char *p){
	struct whentry *ret = MALLOC("whentry", sizeof(struct whentry));
	ret->whe_word = (const char *)xp_strdup(p);
	ret->whe_ldatestr = NULL;
	return ret;
}
	
void
whentry_free(const char *p, struct whentry *whe){
	xp_strfree(whe->whe_word);
	if(whe->whe_ldatestr)
		xp_strfree(whe->whe_ldatestr);
	FREE("whentry", whe);
}

void 
whlookup_load(void)
{
	FILE 		*fp;
	char 		filepath[1024], buf[1024], *p;
	struct 		whentry *whp;

	/* allocate word history map */
	_whmap = whmap_new(13, whmap_hash, whmap_cmp, 
			   xpdic_malloc, xpdic_free);

	snprintf(filepath, 1024, "%s/" WHFILENAME, getenv("HOME"));

	if((fp = fopen(filepath, "r")) == NULL)
		return;

	while(fgets(buf, 1024, fp) != NULL){
		p = index(buf, ',');
		*p++ = '\0';
		whp = whentry_alloc(buf);
		sscanf(p, "%d,%ld\n", &whp->whe_count, &whp->whe_lasttime);
		whp->whe_newtime = whp->whe_lasttime;
		iterator_insert_pair(_whmap, whp->whe_word, whp); 
		mkdatestr(whp);
	}
		
	fclose(fp);
}

void
whlookup_save(void)
{
	FILE 		*fp;
	char		filepath[1024];
	struct 		whentry *whp;
	whmap_iterator 	itr = iterator_first(_whmap);

	snprintf(filepath, 1024, "%s/" WHFILENAME, getenv("HOME"));
	if((fp = fopen(filepath, "w")) == NULL){
		fprintf(stderr, "can't save word-history: ");
		perror(filepath);
		return;
	}

	for(; !iterator_is_end(itr); itr = iterator_next(itr)){
		whp = iterator_data(itr)->second;
		fprintf(fp, "%s,%d,%ld\n", whp->whe_word, whp->whe_count, 
					  whp->whe_newtime);
	}
	fclose(fp);
}

void
whlookup_fin(void)
{
	whlookup_save();
	aggregator_delete_by(_whmap, whentry_free);
}

struct whentry *whlookup_word(const char *p)
{
	char 		word[1024], *q = word;
	whmap_iterator 	itr;
	struct whentry *whp;

	/* tolower */
	while(*p){
		if(*p >= 'A' && *p <= 'Z')
			*q = *p - 'A' + 'a';
		else
			*q = *p;
		++q, ++p;
	}
	*q = '\0';

	itr = iterator_find(_whmap, word);
	if(!iterator_is_end(itr)){
		whp = iterator_data(itr)->second;
		++whp->whe_count;
		whp->whe_lasttime = whp->whe_newtime;
		time(&whp->whe_newtime);

		mkdatestr(whp);
		return whp;
	}

	whp = whentry_alloc(word);
	whp->whe_count = 1;
	time(&whp->whe_newtime);
	mkdatestr(whp);

	iterator_insert_pair(_whmap, whp->whe_word, whp); 
	return whp;
}


#ifdef WHIST_TEST
int
main(int argc, char **argv)
{
	char buf[1024], tbuf1[128], tbuf2[128];
	struct whentry *whp;
	whlookup_load();

	while(fgets(buf, 1024, stdin) != NULL){
		buf[strlen(buf)-1] = '\0';
		if(strlen(buf) == 0)
			continue;

		whp = whlookup_word(buf);

		if(whp->whe_lasttime == 0)
			strcpy(tbuf1, "n/a");
		else {
			strcpy(tbuf1, ctime(&whp->whe_lasttime));
			tbuf1[strlen(tbuf1)-1] = '\0';
		}
		strcpy(tbuf2, ctime(&whp->whe_newtime));
		tbuf2[strlen(tbuf2)-1] = '\0';

/*
		printf("%s: cnt=%d ", whp->whe_word, whp->whe_count);
		printf("last=%s newtime=%s\n", tbuf1, tbuf2);
*/
	}

	whlookup_fin();
	memdump();
	return 0;
}

#endif /* WHIST_TEST */
