/*
 *                      xpobject.h
 * $Id: xpobject.c,v 1.8 2004/01/24 10:44:31 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <ctype.h>
#include <stdio.h>
#include <string.h>

#include "debug.h"
#include "memory.h"
#include "property.h"
#include "bitmap.h"
#include "dictbook.h"
#include "booklist.h"
#include "dictlink.h"
#include "xwindow.h"
#include "stk/stkbase.h"
#include "strset.h"
#include "xobjcache.h"
#include "xpopdic_resource.h"
#include "xpopdic_component.h"
#include "xpopdic_control.h"
#include "xpaction.h"
#include "xpparse.h"

	/* need to draw background at the same time */
#define mkstr(x,a)      xstring_na_new(x, strlen(x), a, 0)
#define mkwstr(x,a)     xwstring_na_new(x, strlen(x), a, 0)

/* table for number to string */
const char *num2str[256] = {
"0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20","21","22","23","24","25","26","27","28","29","30","31","32","33","34","35","36","37","38","39","40","41","42","43","44","45","46","47","48","49","50","51","52","53","54","55","56","57","58","59","60","61","62","63","64","65","66","67","68","69","70","71","72","73","74","75","76","77","78","79","80","81","82","83","84","85","86","87","88","89","90","91","92","93","94","95","96","97","98","99","100","101","102","103","104","105","106","107","108","109","110","111","112","113","114","115","116","117","118","119","120","121","122","123","124","125","126","127","128","129","130","131","132","133","134","135","136","137","138","139","140","141","142","143","144","145","146","147","148","149","150","151","152","153","154","155","156","157","158","159","160","161","162","163","164","165","166","167","168","169","170","171","172","173","174","175","176","177","178","179","180","181","182","183","184","185","186","187","188","189","190","191","192","193","194","195","196","197","198","199","200","201","202","203","204","205","206","207","208","209","210","211","212","213","214","215","216","217","218","219","220","221","222","223","224","225","226","227","228","229","230","231","232","233","234","235","236","237","238","239","240","241","242","243","244","245","246","247","248","249","250","251","252","253","254","255"
};


/* get english string from cache */ 
const char *
xpget_string(struct xpopdic_control *cp, const char *p)
{
	const char *ret;
	strset_iterator itr = iterator_find(cp->xpc->xo_strs, p);

	if(!iterator_is_end(itr))
		return iterator_data(itr);
	ret = xp_strdup(p);
	iterator_insert(cp->xpc->xo_strs, ret);
	return ret;
}

/* get english word object from xobject cache */ 
xobject *
xpget_bodyword(struct xpopdic_control *cp, const char *p)
{
	const char *word = NULL;
	xobject    *ret = NULL;
	xobjcache_iterator itr = iterator_find(cp->xpc->xo_words, p);

	/* cache hit! */
	if(!iterator_is_end(itr))
		return iterator_data(itr)->second;

	/* construct object */
	word = xpget_string(cp, p);

	/* clickable word is required */
	if(cp->xpr->browsable){
		xobject *xo1 = mkstr(word, cp->xpr->xatbody);
		xobject *xo2 = xcmpstpair_new(underline_new(xo1, cp->xpr->xatbody),
					      xo1, 0);
		xobject *xo3 = mkstr(word, cp->xpr->xatclick);
		xactor *xact = xclickable_new(xo1); 

		/* add action/objects */
		xact_register(xact, XSIG_ENTER, wordact.enter, cp, xo2);
		xact_register(xact, XSIG_LEAVE, wordact.leave, cp, xo1);
		xact_register(xact, XSIG_BTNPRESS, wordact.bpress, cp, xo3);
		xact_register(xact, XSIG_BTNRELEASE, wordact.brelease, cp, xo2);
		ret = (xobject *)xact;
	}
	else /* just string */
		ret = xstring_na_new(word, strlen(word), cp->xpr->xatbody, 0);

	iterator_insert_pair(cp->xpc->xo_words, word, ret);
	xobj_incref(ret);
	return ret;
}

/* get linked number object */
xobject *
xpget_listnumber(struct xpopdic_control *cp, unsigned char i)
{
	xobject    *ret = NULL;
	xobjcache_iterator itr = iterator_find(cp->xpc->xo_words, num2str[i]);

	/* cache hit! */
	if(!iterator_is_end(itr))
		return iterator_data(itr)->second;

	else {
		/* clickable is required */
		xobject *xo1 = mkstr(num2str[i], cp->xpr->xatbody);
		xobject *xo2 = xcmpstpair_new(underline_new(xo1, cp->xpr->xatbody), 
					      xo1, 0);
		xobject *xo3 = mkstr(num2str[i], cp->xpr->xatclick);
		xactor *xact = xclickable_new(xo1); 

		/* add action/objects */
		xact_register(xact, XSIG_ENTER, enumact.enter, cp, xo2);
		xact_register(xact, XSIG_LEAVE, enumact.leave, cp, xo1);
		xact_register(xact, XSIG_BTNPRESS, enumact.bpress, cp, xo3);
		xact_register(xact, XSIG_BTNRELEASE, enumact.brelease, cp, xo2);
		ret = (xobject *)xact;
	}

	iterator_insert_pair(cp->xpc->xo_words, num2str[i], ret);
	xobj_incref(ret);
	return ret;
}


static xobject *
xgaiji_new(struct dictbook *book, const char *id, xattr *at, int w, int h)
{
	xobject *ret;
	struct bitmap *bmp = dict_getbitmap(book, id, w, h);

	/* no such font */
	if(!bmp){
		fprintf(stderr, "Gaiji Font(%s): not found in %s.\n", 
			id, book->db_glid);
		return xstring_na_new("?!", 2, at, 0);
	}
	ret = xbitmap_new(bmp, at);
	bitmap_free(bmp);
	return ret;
}


static xobject *
xentptr_new(struct xpopdic_control *cp, const char *p)
{
	xobject *ret = xposmark_new(cp->xpc->xo_ptr);
	xpm_data(ret) = (void *)p;
	return ret;
}

/* XXXX: must be better way */
#define USE_LINK		0
#define USE_NORMAL		1
#define USE_CLICKED		2

static const char *
xpget_next(const char *p_org, struct dictbook *book, 
	   struct xpopdic_control *cp, int w, xobject **o_ret, int uselink)
{
	xobject *xp = NULL;
	int	    n = 0;
	const char *p = p_org;
	const char *p_ret = NULL;
	xattr	*atbody = (uselink == USE_CLICKED? 
				cp->xpr->xatclick: 
				cp->xpr->xatbody);
	xattr	*atwbody = (uselink == USE_CLICKED? 
				cp->xpr->xatwclick: 
				cp->xpr->xatwbody);
	xattr	*atgaiji = (uselink == USE_CLICKED? 
				cp->xpr->xatclick: 
				cp->xpr->xatgaiji);

	if(!p || *p == '\0')
		abort();	

	*o_ret = NULL;

	/* if required width is narrower than width of a space */
	if(w < xobj_width(cp->xpc->xo_space))
		return NULL;

	/* double byte strings */
	else if(xpis_wchr(*p)){
		int wfw = xattr_get_font_width(atwbody);
		p_ret = xpget_wstring(p, &n);

		if(wfw * n > w){
			n = w / wfw;	
			if(n == 0)
				return NULL;
			p_ret = p + (n * 2);
		}
		
		xp = xwstring_new(p, n, atwbody, 1);
		*o_ret = xp;
		return p_ret;
	}
	/* space */
	else if(isspace(*p)){
		*o_ret = cp->xpc->xo_space;
		return p+1;
	}

	/* "<":  */
	else if(xpis_extern(p)){
		char entryid[24];
		int etype = 0;
		p_ret = xpget_entptr(p, entryid, 24, &etype);

		/* gaiji entry */
		if(etype == 1)	/* size is static: XXX */
			*o_ret = xgaiji_new(book, entryid, 
					    atgaiji, 0, 16);
		/* entry pointer */
		else if(etype == 2)
			*o_ret = xentptr_new(cp, p + 1);

		/* just a character "<" */
		else {
			p_ret = p + 1;
			*o_ret = xstring_na_new("<", 1, atbody, 1); 
		}
	}


	/* non-word but ascii */
	else if(!isalpha(*p)){
		int  nlen; 
		p_ret = xpget_noalpha(p, &n);
		xp = xstring_na_new(p, n, atbody, 1);

		/* the width of this obj is small enough */
		if(xobj_width(xp) <= w){
			*o_ret = xp;
			return p_ret;
		}

		/* must divide */
		nlen = (int)((double)n * (double)((double)w/
				(double)(xobj_width(xp))));

		/* free */
		xobj_free(xp);

		/* should give up on this line */
		if(nlen < 2)
			return NULL;

		*o_ret = xstring_na_new(p, n, atbody, 1);
		return p + n;
	}

	/* a word */
	else if(isalpha(*p)){
		char wbuf[256];
		const char *wp;
		p_ret = xpget_word(p, &n);

		wbuf[0] = '\0';
		strncpy(wbuf, p, (n < 255)? n: 255);
		wbuf[(n < 255)? n: 255] = '\0';

		/* get string from cache */
		wp = xpget_string(cp, wbuf);

		/* there is not enough room */
		if(xattr_get_str_width(atbody, wp, n) > w)
			return NULL;

		if(uselink == USE_LINK)
			*o_ret = xpget_bodyword(cp, wp);
		else
			*o_ret = mkstr(wp, atbody);
	}

	return p_ret;
}

const char *
xpget_next_object(const char *p, struct dictbook *book, 
		 struct xpopdic_control *cp, int w, xobject **ret)
{
	return xpget_next(p, book, cp, w, ret, USE_LINK);
}

static xobject *
build_clickable(xobject *a, xobject *c, const char *entryid, struct xpopdic_control *cp)
{
	xactor *ret = xclickable_new(xposmark_new(a));
	xobject *b;

	/* save here */
	xpm_data(ret->xa_object) = (void *)entryid;

	/* compose containers first */
	xcmp_compose(a);
	xcmp_compose(c);
	
	/* make b with a and underline. */
 	b = xcmpstpair_new(underline_new(a, cp->xpr->xatbody), a, XCRULE_CENTER);
	xcmp_compose(b);

	xact_register(ret, XSIG_ENTER, etitleact.enter, cp, b);
	xact_register(ret, XSIG_LEAVE, etitleact.leave, cp, a);
	xact_register(ret, XSIG_BTNPRESS, etitleact.bpress, cp, c);
	xact_register(ret, XSIG_BTNRELEASE, etitleact.brelease, cp, b);

	return (xobject *)ret;
}

static xobject *
xpget_list_entry_short(const char *title, const char *entryid, 
		       struct dictbook *book, struct xpopdic_control *cp, int w);
static xobject *
xpget_list_entry_long(const char *title, const char *entryid, 
		       struct dictbook *book, struct xpopdic_control *cp, int w);
xobject *
xpget_list_entry(const char *title, const char *entryid, 
		 struct dictbook *book, struct xpopdic_control *cp, int w)
{
	/* estimate width required for this title and choose either of two ways. */
	int	bw = xattr_get_font_width(cp->xpr->xatbody);
		/* 10pt: just margin for wide characters */
	int	tw = bw * strlen(title) + 10;	

	/* enough space */
	if(w > tw)
		return xpget_list_entry_short(title, entryid, book, cp, w);

	return xpget_list_entry_long(title, entryid, book, cp, w);
}


static xobject *
xpget_list_entry_short(const char *title, const char *entryid, 
		       struct dictbook *book, struct xpopdic_control *cp, int w)
{
	int	wleft 	= w;
	const char *p = title, *np;
	const char *dictname = book->db_name;
	int	dw, dlen;
	xobject *vl 	= NULL;
	xobject *ret 	= xhlayout_new(w,  -1, 0, XCRULE_LEFT);
	xobject *t 	= xhlayout_new(-1, -1, 0, XCRULE_LEFT);
	xobject *lt 	= xhlayout_new(-1, -1, 0, XCRULE_LEFT);
	xobject *xo, *xlo, *clp;


	while(*p != '\0'){
		np = xpget_next(p, book, cp, wleft, &xo, USE_NORMAL);
	
		/* no more room */
		if(np == NULL){
			/* estimation is wrong. give up! */
			break;
		}
		np = xpget_next(p, book, cp, wleft, &xlo, USE_CLICKED);
		xcont_add(t, xo, XCRULE_CENTER);
		xcont_add(lt, xlo, XCRULE_CENTER);
		wleft -= xobj_width(xo);
		p = np;
	}

	clp = build_clickable(t, lt, entryid, cp);
	xcmp_compose(clp);
	xcont_add(ret, clp, XCRULE_CENTER);

	/* width of dictionary name */
	dlen = strlen(dictname) / 2;
	dw = xattr_get_str_width(cp->xpr->xatdict, dictname, dlen);

	if(wleft > dw){
		xobject *xo = xwstring_na_new(dictname, dlen,
					   cp->xpr->xatdict, 0);
		xcont_add(ret, cp->xpc->xo_expdr, XCRULE_CENTER);
		xcont_add(ret, xo, XCRULE_CENTER);

		if(vl){
			xcont_add(vl, ret, XCRULE_LEFT);
			ret = vl;
		}
	}
	else {
		xobject *xo;
		int fw = xattr_get_font_width(cp->xpr->xatdict);
		int nc = wleft / fw;

		/* at least 2 characters on the current line */
		if(nc < 2)
			nc = 0;

		/* at least 2 characters on the next line */
		else if(dlen - nc < 2)
			nc = dlen - 2;

		if(nc){
			xo = xwstring_na_new(dictname, nc,
					   	   cp->xpr->xatdict, 0);
			xcont_add(ret, cp->xpc->xo_expdr, XCRULE_CENTER);
			xcont_add(ret, xo, XCRULE_CENTER);
		}
		if(!vl)
			vl = xvlayout_new(w, -1, 0, XCRULE_TOP);

		xcont_add(vl, ret, 0);

		xo = xwstring_na_new(dictname + (nc * 2), dlen - nc,
				     cp->xpr->xatdict, 0);
		xcont_add(vl, xo, XCRULE_RIGHT);
		ret = vl;
	}

	return ret;
}

static xobject *
xpget_list_entry_long(const char *title, const char *entryid, 
		       struct dictbook *book, struct xpopdic_control *cp, int w)
{
	return xpget_list_entry_short(title, entryid, book, cp, w);

	/* XXXXXXXXXXX  must be implemented multiple link string */
#if 0
	int	wleft 	= w;
	const char *p = title, *np;
	const char *dictname = book->db_name;
	int	dw, dlen;
	xobject *vl 	= NULL;
	xobject *ret 	= xhlayout_new(w,  -1, 0, XCRULE_LEFT);
	xobject *t 	= xhlayout_new(-1, -1, 0, XCRULE_LEFT);
	xobject *lt 	= xhlayout_new(-1, -1, 0, XCRULE_LEFT);
	xobject *xo, *xlo, *clp;


	while(*p != '\0'){
		np = xpget_next(p, book, cp, wleft, &xo, USE_NORMAL);
	
		/* no more room: need vlayout */
		if(np == NULL){
			if(!vl)
				vl = xvlayout_new(w, -1, 0, XCRULE_TOP);

			/* make clickable object */
			clp = build_clickable(t, lt, entryid, vl, cp);
			xcont_add(vl, clp, XCRULE_LEFT);

			t = xhlayout_new(-1, -1, 0, XCRULE_CENTER);
			lt = xhlayout_new(-1, -1, 0, XCRULE_CENTER);
			wleft = w;
			continue;
		}
		np = xpget_next(p, book, cp, wleft, &xlo, USE_CLICKED);
		xcont_add(t, xo, XCRULE_CENTER);
		xcont_add(lt, xlo, XCRULE_CENTER);
		wleft -= xobj_width(xo);
		p = np;
	}

	clp = build_clickable(t, lt, entryid, vl, cp);
	xcmp_compose(clp);
	xcont_add(ret, clp, XCRULE_CENTER);

	/* width of dictionary name */
	dlen = strlen(dictname) / 2;
	dw = xattr_get_str_width(cp->xpr->xatdict, dictname, dlen);

	if(wleft > dw){
		xobject *xo = xwstring_na_new(dictname, dlen,
					   cp->xpr->xatdict, 0);
		xcont_add(ret, cp->xpc->xo_expdr, XCRULE_CENTER);
		xcont_add(ret, xo, XCRULE_CENTER);

		if(vl){
			xcont_add(vl, ret, XCRULE_LEFT);
			ret = vl;
		}
	}
	else {
		xobject *xo;
		int fw = xattr_get_font_width(cp->xpr->xatdict);
		int nc = wleft / fw;

		/* at least 2 characters on the current line */
		if(nc < 2)
			nc = 0;

		/* at least 2 characters on the next line */
		else if(dlen - nc < 2)
			nc = dlen - 2;

		if(nc){
			xo = xwstring_na_new(dictname, nc,
					   	   cp->xpr->xatdict, 0);
			xcont_add(ret, cp->xpc->xo_expdr, XCRULE_CENTER);
			xcont_add(ret, xo, XCRULE_CENTER);
		}
		if(!vl)
			vl = xvlayout_new(w, -1, 0, XCRULE_TOP);

		xcont_add(vl, ret, 0);

		xo = xwstring_na_new(dictname + (nc * 2), dlen - nc,
				     cp->xpr->xatdict, 0);
		xcont_add(vl, xo, XCRULE_RIGHT);
		ret = vl;
	}

	return ret;
#endif
}

void
xpput_line(const char *p_org, xobject *body, int w, struct dictbook *d,
	   struct xpopdic_control *cp)
{
	int wleft = w;
	const char *p = p_org, *np;
	xobject *hlp = xhlayout_new(w, -1, 0, XCRULE_LEFT);

	while(*p){
		xobject *xo;
		np = xpget_next_object(p, d, cp, wleft, &xo);

		if(np == NULL){
			xcont_add(body, hlp, XCRULE_CENTER);
			hlp = xhlayout_new(w, -1, 0, XCRULE_LEFT);
			wleft = w;
			continue;
		}
		xcont_add(hlp, xo, XCRULE_CENTER);
		p = np;
		wleft -= xobj_width(xo);
	}
	xcont_add(body, hlp, XCRULE_CENTER);
}

