/*
 *			xpobject.h
 * $Id: xpobject.h,v 1.3 2004/01/23 02:45:27 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#ifndef _XP_OBJECT_H_
#define _XP_OBJECT_H_

/*
 * xpget_next_object():
 *	- xpget_next_object() parses string starting pointer p,
 *	constructs an object for the string, set the object into 
 *	indicated pointer and return next string pointer.
 *	- if return NULL, it means that current width 'w' is not 
 *	enough to fit the next object. So the caller must next line
 *	to contain next object.
 */
const char *
xpget_next_object(const char *p, struct dictbook *,
		  struct xpopdic_control *cp, int w, xobject **ret);

/*
 * xpget_list_entry():
 *	- xpget_list_entry() builds an object to show entry on a list 
 *	view.
 */
xobject *
xpget_list_entry(const char *entrytitle, const char *entryid, 
		 struct dictbook *, struct xpopdic_control *cp, int w);

/* 
 * xpget_string(struct xpopdic_control *cp, const char *p);
 *	- get allocated string if it is found in string cache.
 *	- if it's not found, allocates the string starting p, 
 *	puts it into the cache and returns it.
 */
const char *
xpget_string(struct xpopdic_control *cp, const char *p);

/* 
 * xpput_line(const char *p, xobject *vl, int w, struct xpopdic_control *cp);
 *	- divides a line starting with 'p' into multiple objects and 
 *	constructs a sequece of objects.
 *	- puts them into vl to fit the required width 'w'.
 *	- this is called by xptop or xptrans to construct a body field. 
 */
void 
xpput_line(const char *p, xobject *vl, int w, struct dictbook *, 
	   struct xpopdic_control *cp);


/* 
 * xpget_bodyword(struct xpopdic_control *cp, const char *p);
 *	- get a word object in body field. If it is found in
 *	object cache, it is just returned. Otherwise, allocates 
 *	the object, puts it into the cache and returns it.
 *	- the object is clickable if the option "browsable" is 
 *	set. Otherwise just xstring..
 */
xobject *
xpget_bodyword(struct xpopdic_control *cp, const char *p);

/* 
 * xpget_listnumuber(struct xpopdic_control *cp, unsigned char i);
 *	- get a list number object in list view. If it is found in
 *	object cache, it is just returned. Otherwise, allocates 
 *	the object, puts it into the cache and returns it.
 *	- the object is clickable if the option "browsable" is 
 *	set. Otherwise just xstring..
 */
xobject *
xpget_listnumber(struct xpopdic_control *cp, unsigned char i);


/*
 * 	xobject *xpmk_body(int height, xobject **ret_body);
 *		- return vlayout for body and set top object ob body. 
 */
xobject *
xpmk_body(xobject *, int height, xobject **ret_body);


/* array to convert number to string */
extern const char *num2str[256];
#endif /* _XP_OBJECT_H_ */

