/*
 *                      xppage_layout.c
 * $Id: xppage_layout.c,v 1.6 2004/02/12 00:45:58 hiddy Exp $
 *
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <time.h>
#include <stdio.h>
#include <string.h>
#include "memory.h"
#include "bitmap.h"
#include "wordhist.h"
#include "dictbook.h"
#include "booklist.h"
#include "dictlink.h"
#include "strlist.h"
#include "strset.h"
#include "xobjcache.h"
#include "xwindow.h"
#include "stk/stkbase.h"
#include "xpopdic_resource.h"
#include "xpopdic_component.h"
#include "xpopdic_control.h"
#include "xpobject.h"
#include "xpaction.h"
#include "xppage.h"

#define mkstr(x,y)	xstring_na_new(x, strlen(x), y, 0)
#define xo_abs(x)       ((x) < 0 ? -(x) : (x))
#define xo_max(x, y)    ((x) < (y) ? (y) : (x))
#define xo_min(x, y)    ((x) < (y) ? (x) : (y))

xobject *
xppage_top_border(struct xpopdic_control *cp, struct dictnode *dnode);
xobject *
xppage_top_body(struct xpopdic_control *cp, struct dictnode *dnode, 
                unsigned int w, unsigned int h);
xobject *
xppage_list_border(struct xpopdic_control *cp, struct dictnode *dnode);
xobject *
xppage_list_body(struct xpopdic_control *cp, struct dictnode *dnode, 
                unsigned int w, unsigned int h);
xobject *
xppage_trans_border(struct xpopdic_control *cp, struct dictnode *dnode);
xobject *
xppage_trans_body(struct xpopdic_control *cp, struct dictnode *dnode, 
                unsigned int w, unsigned int h);


static xobject *
construct_top(struct xpopdic_control *cp, struct dictnode *dnode)
{
	/* make top line of page */
	xobject *top = xhlayout_new(cp->xpr->width, -1, 0, XCRULE_CENTER);
	xcont_add(top, cp->xpc->xo_status, XCRULE_CENTER);
	xcont_add(top, cp->xpc->xo_expdr, XCRULE_CENTER);

	if(dnode && dl_has_prev(dnode))
		xcont_add(top, cp->xpc->xo_prev, XCRULE_TOP);
	if(dnode && dl_has_up(dnode))
		xcont_add(top, cp->xpc->xo_up, XCRULE_TOP);
	if(dnode && dl_has_next(dnode))
		xcont_add(top, cp->xpc->xo_next, XCRULE_TOP);
	xcont_add(top, cp->xpc->xo_quit, XCRULE_TOP);
	xcont_add(top, xspace_new(1,xobj_height(cp->xpc->xo_quit)+2), XCRULE_CENTER);
	xcmp_compose(top);

	return top;
}

static xobject *
construct_border(struct xpopdic_control *cp, struct dictnode *dnode)
{
	xobject *border;
	
	if(!dnode)
		border = xppage_top_border(cp, dnode);
	else if(dnode->dn_type == DNTYPE_LIST)
		border = xppage_list_border(cp, dnode);
	else if(dnode->dn_type == DNTYPE_TRANS)
		border = xppage_trans_border(cp, dnode);
	xcmp_compose(border);
	return border;
}

static xobject *
construct_body(struct xpopdic_control *cp, struct dictnode *dnode, 
	       int rest_height, xobject **bodyp)
{
	int body_width = cp->xpr->width - cp->xpr->bsidemargin * 2;
	int body_height = rest_height - cp->xpr->btbmargin * 2;
	int bodybg_width = body_width;
	int bodybg_height = body_height;
	int bodyfg_width = bodybg_width - 2 * 2;
	int bodyfg_height = bodybg_height - 2 * 2;
	xobject *fgbody;
	xobject *ret = xcomposite_new(body_width, body_height); 
	xobject *pane = xrect_new(body_width, body_height, 0, 
				  cp->xpr->xatbodybdc);  
	xobject *bgbody = xrect_new(body_width, bodybg_height, 1, 
				    cp->xpr->xatbodybgc);
	xobject *vfgbody = xvlayout_new(body_width, -1, 0, XCRULE_TOP);

	/* prevent to change bgbody's color to inactive color */ 
	xobj_flags(bgbody) |= XOBJFLAG_ALWACTIVE;

	if(!dnode)
		fgbody = xppage_top_body(cp, dnode, bodyfg_width, bodyfg_height);
	else if(dnode->dn_type == DNTYPE_LIST)
		fgbody = xppage_list_body(cp, dnode, bodyfg_width, bodyfg_height);
	else if(dnode->dn_type == DNTYPE_TRANS)
		fgbody = xppage_trans_body(cp, dnode, bodyfg_width, bodyfg_height);
	*bodyp = fgbody;
	xcmp_compose(fgbody);

	if(bodyfg_height < xobj_height(fgbody)){
                /* need scroll */
		xobject *scrl = xscroll_new_bg(fgbody, xobj_width(fgbody), 
					       bodyfg_height, cp->xpr->xatbodybgc);
		xactor *ascrl = xascroll_new((xscroll *)scrl);
		fgbody = (xobject *)ascrl;
	}
	else
		fgbody = xframe_new_bg(fgbody, xobj_width(fgbody), 
					xobj_height(fgbody), cp->xpr->xatbodybgc);
	xcont_add(vfgbody, xspace_new(0, 2), 0);
	xcont_add(vfgbody, fgbody, XCRULE_CENTER);
	xcmp_compose(vfgbody);

	xcont_add(ret, pane, XCRULE_CENTER);
	xcont_add(ret, vfgbody, XCRULE_TOP);
	xcont_add(ret, bgbody, XCRULE_CENTER);
	xcmp_compose(ret);
	return ret;
}

static xobject *
construct_page(struct xpopdic_control *cp, struct dictnode *dnode, xobject **bodyp)
{
	int theight = 0;	/* total height */
	xobject *vlayout = xvlayout_new(cp->xpr->width, cp->xpr->height,
					0, XCRULE_TOP);
	xobject *top = construct_top(cp, dnode);
	xobject *title;
	xobject *border;
	xobject *btmargin;
	xobject *body;

	/* add top */
	xcont_add(vlayout, top, XCRULE_CENTER);
	theight += xobj_height(top);

	/* add Title */
	if(!cp->xpr->notitle){
		if(dnode)
			title = mkstr(dnode->dn_whist->whe_word, cp->xpr->xattitle);
		else
			title = mkstr("xpopdic", cp->xpr->xattitle);
		xcont_add(vlayout, title, XCRULE_CENTER);
		theight += xobj_height(title);
	}

	/* add border */
	if(!cp->xpr->nosubinfo){
		border = construct_border(cp, dnode);
		xcont_add(vlayout, border, XCRULE_CENTER);
		theight += xobj_height(border);
	}

	/* add top margin between border and body */
	btmargin = xspace_new(0, cp->xpr->btbmargin);
	xcont_add(vlayout, btmargin, XCRULE_CENTER);

	/* construct body and put it into vlayout */
	body = construct_body(cp, dnode, cp->xpr->height - theight, bodyp);
	xcont_add(vlayout, body, XCRULE_CENTER);

	return vlayout;
}


struct xobject *
xppage_build(struct xpopdic_control *cp, struct dictnode *dnode)
{
	xobject *body;
	xobject *page = construct_page(cp, dnode, &body);
	xactor *key = xkeyaction_new(page);

	xact_register(key, XSIG_KEYPRESS, xppage_keyaction, cp, NULL);
	xact_register(key, XSIG_KEYRELEASE, xppage_keyaction, cp, NULL);
	xobj_disable(cp->xpc->xo_search);
	return xppage_new((xobject *)key, cp->xpc->xo_search, body);
}

