/*
 *			xpstatus.c
 * $Id: xpstatus.c,v 1.3 2004/01/22 10:17:38 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#define _XACTOR_INTERNAL

#include <stdio.h>
#include <strings.h>
#include "memory.h"
#include "bitmap.h"
#include "xwindow.h"
#include "stk/stkbase.h"
#include "xpopdic_resource.h"
#include "xpstatus.h"

/*
 *    XPopdic status bar  
 *	Treated signal: 
 *		- XSIG_STAT_NORMAL
 *		- XSIG_STAT_WARN
 *		- XSIG_STAT_LINK
 *		- XSIG_STAT_UP
 *
 *	Structure of status bar
 *	status
 *	  |
 *	hlayout [ 'width of window' - 'sizeof 4 icons and margin' ]
 *	  |
 *	  +--  xspace <3 x 3 >
 *	  |
 *	  +--  xselector 
 *	  |        0:  xbitmap<glass1.bmp>	normal
 *	  |        1:  xbitmap<glass2.bmp>	link
 *	  |        2:  xbitmap<glass3.bmp>	up
 *	  |        3:  xbitmap<glass4.bmp>	warn
 *	  |
 *	  +--  xspace <3 x 3 >
 *	  |	
 *	  +--  xstring	<-- rewritten by signals.
 */
#define XPS_NORMAL 	0
#define XPS_LINK 	1
#define XPS_UP 		2
#define XPS_WARN 	3
struct xpstatus {
	struct xactor 	xps_base;
	xstr		*xps_str;	 /* current status mesg */
	char		xps_normal[128]; /* normal message cache */
	char		xps_current[128];/* current message cache */
	xobject 	*xps_sel;
};

#define xo_max(x, y)    ((x) < (y) ? (y) : (x))
#define xo_min(x, y)    ((x) < (y) ? (x) : (y))

static void
xps_free(struct xobject *o)
{
	xobj_free(((struct xactor *)o)->xa_object);
	FREE("xpstatus", o);
}

static void
xps_dump(struct xobject *o, FILE *fp, int off)
{
	struct xactor *r = (struct xactor *)o;
	char idt[1024], buf[1024], info[1024], *p = idt;

	xod_indent(idt, off);
	xobj_info(o, info, 1024);
	
	snprintf(buf, 1024, "%s%s<Xpopdic Status>: %s{\n", 
		 idt, xobj_typestr(o), info);
	fprintf(fp, "%s", buf);

	while(*p)
		*p++ = ' ';

	fprintf(fp, "%s Main:\n", idt); 
	xobj_dump(r->xa_object, fp, off + 1);
	fprintf(fp, "%s}\n", idt);
}


void
xps_draw(struct xobject *o, int x, int y, xwindow *xwp, 
	    int rx, int ry, unsigned int rw, unsigned int rh, int f)
{
	struct xactor *xrp = (struct xactor *)o;

	xw_clear_region(xwp, x, y, xobj_width(o), xobj_height(o));

	xobj_draw_rect(xrp->xa_object, x, y, xwp, 0, 0, xobj_width(o), 
		       xobj_height(o), 
		       xobj_is_active(o)? f : f | XOBJDRAW_INACTIVE);

	/*
	 	xobj_draw_rect(xrp->xa_object, x, y, xwp, rx, ry, rw, rh, 
		 		xobj_is_active(o)? f : f | XOBJDRAW_INACTIVE);
	*/
}


static void
xps_compose(struct xcomposer *o)
{
	struct xactor *r = (struct xactor *)o;
	if(xobj_is_composer(r->xa_object))
		xcmp_compose(r->xa_object);
}

static int
xps_acceptable(struct xactor *r, xsignal s)
{
	switch(xsignal_type(s)){
	case XSIG_STAT_NORMAL:
	case XSIG_STAT_LINK:
	case XSIG_STAT_UP:
	case XSIG_STAT_WARN:
		return 1;
	}
	return 0;

}

static struct xaction *
xps_getact(struct xactor *r, xsignal s)
{
	return NULL;	/* no action is registered */
}

static int
xps_register(struct xactor *r, xsignal s, xaction_t a, 
	   void *p, struct xobject *o)
{
	return 0;  /* nothing can be registered */
}


static void
xps_receive(struct xactor *act, xpoint pos, xsignal sig, struct xsigarg *sigarg, 
	    struct xobject *root, xwindow *w)
{
	struct xpstatus *c = (struct xpstatus *)act;
	struct xps_sigarg *a = (struct xps_sigarg *)sigarg;

	switch(xsignal_type(sig)){
	case XSIG_STAT_NORMAL:
		xsel_select(c->xps_sel, XPS_NORMAL);	
		if(a->msg != NULL){
			/* cache normal message */
			strncpy(c->xps_normal, a->msg, sizeof(c->xps_normal));
			c->xps_normal[a->nchars] = '\0';
		}
		break;
	case XSIG_STAT_LINK:
		xsel_select(c->xps_sel, XPS_LINK);	
		break;
	case XSIG_STAT_UP:
		xsel_select(c->xps_sel, XPS_UP);
		break;
	case XSIG_STAT_WARN:
		xsel_select(c->xps_sel, XPS_WARN);
		break;
	default:
		return;	/* unknown signal */
	}

	if(a->msg){
		/* little tricky to reduce memory allocation */
		strncpy(c->xps_current, a->msg, sizeof(c->xps_current));
		c->xps_current[a->nchars] = '\0';
		c->xps_str->str = c->xps_current;
		c->xps_str->nchars = a->nchars;
	}
	else {
		c->xps_str->str = c->xps_normal;
		c->xps_str->nchars = strlen(c->xps_normal);
	}
	xobj_width(c->xps_str) = xattr_get_str_width(xpmt_attr(c->xps_str), 
					     c->xps_str->str, c->xps_str->nchars);
	xobj_redraw_object(root, act, pos.x, pos.y, w, 0);
}

struct xactor *
xpstatus_new(struct xpopdic_resource *rp)
{
	int stwidth = rp->width - (rp->up->bm_width * 4) - 30;
	int stheight = xo_max(rp->glass1->bm_height, 
			      xattr_get_font_height(rp->xatstat));
	struct xobject *sel = xselector_new(4, rp->glass1->bm_width, 
						 rp->glass1->bm_height);
	struct xobject *cnt = xhlayout_new(stwidth, stheight, 0, XCRULE_LEFT);
	struct xpstatus *c = MALLOC("xpstatus", sizeof(struct xpstatus));
	struct xactor *ret = (struct xactor *)c;
	struct xobject *spc = xspace_new(3, 3);


	/* setup object selector */
	xsel_set(sel, XPS_NORMAL, xbitmap_new(rp->glass1, rp->xatglass1)); 
	xsel_set(sel, XPS_LINK, xbitmap_new(rp->glass2, rp->xatglass2)); 
	xsel_set(sel, XPS_UP, xbitmap_new(rp->glass3, rp->xatglass3)); 
	xsel_set(sel, XPS_WARN, xbitmap_new(rp->glass4, rp->xatglass4)); 
	xsel_select(sel, 0);		/* default: normal */
	c->xps_sel = sel;

	/* construct horizontal layout */
	xcont_add(cnt, spc, XCRULE_CENTER);
	xcont_add(cnt, sel, XCRULE_CENTER);
	xcont_add(cnt, spc, XCRULE_CENTER);
	strcpy(c->xps_normal, "Xpopdic starting.");
	strcpy(c->xps_current, "Xpopdic starting.");
	c->xps_str = (xstr *)xstring_na_new(c->xps_normal, strlen(c->xps_normal), 
					    rp->xatstat, 0);
	xcont_add(cnt, (xobject *)c->xps_str, XCRULE_CENTER); 

	xactor_init(ret, cnt, xps_draw, xps_free, xps_dump, xps_compose, 
		      xactor_find, xactor_relocate);

	ret->xa_acceptable = xps_acceptable;
	ret->xa_getaction = xps_getact;
	ret->xa_register = xps_register;
	ret->xa_receive = xps_receive;
	return ret;
}
